from unittest.mock import patch

from django.urls import reverse

from standingsrequests.views.effective_requests import effective_requests_data

from ..my_test_data import (
    TestViewPagesBase,
    esi_get_corporations_corporation_id,
    esi_post_universe_names,
)
from ..utils import json_response_to_dict_2

HELPERS_EVECORPORATION_PATH = "standingsrequests.helpers.evecorporation"


@patch(HELPERS_EVECORPORATION_PATH + ".cache")
@patch(HELPERS_EVECORPORATION_PATH + ".esi")
class TestEffectiveRequestsData(TestViewPagesBase):
    def test_request_character(self, mock_esi, mock_cache):
        # given
        alt_id = self.alt_character_1.character_id
        self._create_standing_for_alt(self.alt_character_1)
        request = self.factory.get(reverse("standingsrequests:effective_requests_data"))
        request.user = self.user_manager
        my_view_without_cache = effective_requests_data.__wrapped__

        # when
        response = my_view_without_cache(request)

        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_dict_2(response, "contact_id")
        expected = {alt_id}
        self.assertSetEqual(set(data.keys()), expected)
        self.maxDiff = None

        data_alt_1 = data[self.alt_character_1.character_id]
        expected_alt_1 = {
            "contact_id": 1007,
            "contact_name": "James Gordon",
            "corporation_name": "Metro Police",
            "corporation_ticker": "MP",
            "alliance_name": "",
            "has_scopes": True,
            "state": "Member",
            "main_character_name": "Peter Parker",
            "action_by": self.user_manager.username,
            "labels_str": "",
        }
        self.assertPartialDictEqual(data_alt_1, expected_alt_1)

    def test_request_corporation(self, mock_esi, mock_cache):
        # given
        mock_Corporation = mock_esi.client.Corporation
        mock_Corporation.get_corporations_corporation_id.side_effect = (
            esi_get_corporations_corporation_id
        )
        mock_esi.client.Universe.post_universe_names.side_effect = (
            esi_post_universe_names
        )
        mock_cache.get.return_value = None
        alt_id = self.alt_corporation.corporation_id
        self._create_standing_for_alt(self.alt_corporation)
        request = self.factory.get(reverse("standingsrequests:effective_requests_data"))
        request.user = self.user_manager
        my_view_without_cache = effective_requests_data.__wrapped__

        # when
        response = my_view_without_cache(request)

        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_dict_2(response, "contact_id")
        expected = {alt_id}
        self.assertSetEqual(set(data.keys()), expected)
        self.maxDiff = None

        expected_alt_1 = {
            "contact_id": 2004,
            "contact_name": "Metro Police",
            "corporation_name": "Metro Police",
            "corporation_ticker": "MP",
            "alliance_name": "",
            "has_scopes": True,
            "state": "Member",
            "main_character_name": "Peter Parker",
            "action_by": self.user_manager.username,
            "labels_str": "",
        }
        self.assertPartialDictEqual(data[alt_id], expected_alt_1)



from datetime import timedelta
from unittest.mock import patch

from django.contrib.auth.models import User
from django.core.cache import cache
from django.test import override_settings
from django.urls import reverse
from django.utils.timezone import now
from django_webtest import WebTest

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.notifications.models import Notification
from allianceauth.tests.auth_utils import AuthUtils
from app_utils.testing import add_character_to_user

from .. import tasks
from ..core import ContactType
from ..models import (
    Contact,
    ContactSet,
    RequestLogEntry,
    StandingRequest,
    StandingRevocation,
)
from .my_test_data import (
    TEST_STANDINGS_ALLIANCE_ID,
    TEST_STANDINGS_API_CHARID,
    create_contacts_set,
    create_eve_objects,
    create_standings_char,
    esi_get_corporations_corporation_id,
    esi_post_characters_affiliation,
    esi_post_universe_names,
    load_eve_entities,
)

CORE_PATH = "standingsrequests.core"
MODELS_PATH = "standingsrequests.models"
MANAGERS_PATH = "standingsrequests.managers"
TASKS_PATH = "standingsrequests.tasks"
TEST_REQUIRED_SCOPE = "publicData"
HELPERS_EVECORPORATION_PATH = "standingsrequests.helpers.evecorporation"


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
@patch(
    MODELS_PATH + ".SR_REQUIRED_SCOPES",
    {"Member": [TEST_REQUIRED_SCOPE], "Blue": [], "": []},
)
@patch(CORE_PATH + ".STANDINGS_API_CHARID", TEST_STANDINGS_API_CHARID)
@patch(MANAGERS_PATH + ".SR_NOTIFICATIONS_ENABLED", True)
@patch(CORE_PATH + ".STR_ALLIANCE_IDS", [TEST_STANDINGS_ALLIANCE_ID])
class TestMainUseCases(WebTest):

    csrf_checks = False

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        create_standings_char()
        create_eve_objects()
        load_eve_entities()
        # State is alliance, all members can add standings
        cls.member_state = AuthUtils.get_member_state()
        perm = AuthUtils.get_permission_by_name(StandingRequest.REQUEST_PERMISSION_NAME)
        cls.member_state.permissions.add(perm)
        # Requesting user
        cls.main_character_1 = EveCharacter.objects.get(character_id=1002)
        cls.user_requestor = AuthUtils.create_user(cls.main_character_1.character_name)
        add_character_to_user(
            cls.user_requestor,
            cls.main_character_1,
            is_main=True,
            scopes=[
                TEST_REQUIRED_SCOPE,
            ],
        )
        cls.member_state.member_characters.add(cls.main_character_1)
        cls.alt_character_1 = EveCharacter.objects.get(character_id=1007)
        add_character_to_user(
            cls.user_requestor,
            cls.alt_character_1,
            scopes=[
                TEST_REQUIRED_SCOPE,
            ],
        )
        cls.alt_corporation = EveCorporationInfo.objects.get(
            corporation_id=cls.alt_character_1.corporation_id
        )
        cls.alt_character_2 = EveCharacter.objects.get(character_id=1008)
        add_character_to_user(
            cls.user_requestor,
            cls.alt_character_2,
            scopes=[
                TEST_REQUIRED_SCOPE,
            ],
        )
        # Standing manager
        cls.main_character_2 = EveCharacter.objects.get(character_id=1001)
        cls.user_manager = AuthUtils.create_user(cls.main_character_2.character_name)
        add_character_to_user(
            cls.user_manager,
            cls.main_character_2,
            is_main=True,
            scopes=[
                TEST_REQUIRED_SCOPE,
            ],
        )
        cls.member_state.member_characters.add(cls.main_character_2)
        cls.user_manager = AuthUtils.add_permission_to_user_by_name(
            StandingRequest.REQUEST_PERMISSION_NAME, cls.user_manager
        )
        cls.user_manager = AuthUtils.add_permission_to_user_by_name(
            "standingsrequests.affect_standings", cls.user_manager
        )

    @patch(TASKS_PATH + ".ContactSet.objects.create_new_from_api")
    def _process_standing_requests(self, mock_create_new_from_api):
        mock_create_new_from_api.return_value = self.contact_set
        tasks.standings_update()

    def _set_standing_for_alt_in_game(self, alt: object) -> None:
        if isinstance(alt, EveCharacter):
            Contact.objects.update_or_create(
                contact_set=self.contact_set,
                eve_entity_id=alt.character_id,
                defaults={"standing": 10},
            )
        elif isinstance(alt, EveCorporationInfo):
            Contact.objects.update_or_create(
                contact_set=self.contact_set,
                eve_entity_id=alt.corporation_id,
                defaults={"standing": 10},
            )
        else:
            raise NotImplementedError()

        self.contact_set.refresh_from_db()

    def _create_standing_for_alt(self, alt: object) -> StandingRequest:
        if isinstance(alt, EveCharacter):
            contact_id = alt.character_id
            contact_type_id = ContactType.character_id
        elif isinstance(alt, EveCorporationInfo):
            contact_id = alt.corporation_id
            contact_type_id = ContactType.corporation_id
        else:
            raise NotImplementedError()

        return StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=contact_id,
            contact_type_id=contact_type_id,
            action_by=self.user_manager,
            action_date=now() - timedelta(days=1, hours=1),
            is_effective=True,
            effective_date=now() - timedelta(days=1),
        )

    def _remove_standing_for_alt_in_game(self, alt: object) -> None:
        if isinstance(alt, EveCharacter):
            Contact.objects.get(
                contact_set=self.contact_set, eve_entity_id=alt.character_id
            ).delete()
        elif isinstance(alt, EveCorporationInfo):
            Contact.objects.get(
                contact_set=self.contact_set, eve_entity_id=alt.corporation_id
            ).delete()
        else:
            raise NotImplementedError()

        self.contact_set.refresh_from_db()

    def _parse_contacts_data(self, response, key: str):
        return {row["contact_id"]: row for row in response.context.dicts[3][key]}

    def _setup_mocks(self, mock_esi, mock_esi_manager):
        mock_esi.client.Corporation.get_corporations_corporation_id.side_effect = (
            esi_get_corporations_corporation_id
        )
        mock_esi_manager.client.Corporation.get_corporations_corporation_id.side_effect = (
            esi_get_corporations_corporation_id
        )
        mock_esi_manager.client.Character.post_characters_affiliation.side_effect = (
            esi_post_characters_affiliation
        )
        mock_esi.client.Universe.post_universe_names.side_effect = (
            esi_post_universe_names
        )

    def setUp(self) -> None:
        cache.clear()
        ContactSet.objects.all().delete()
        self.contact_set = create_contacts_set()
        StandingRequest.objects.all().delete()
        StandingRevocation.objects.all().delete()
        Notification.objects.all().delete()

        # requestor as permission
        self.user_requestor = AuthUtils.add_permission_to_user_by_name(
            StandingRequest.REQUEST_PERMISSION_NAME, self.user_requestor
        )

    @patch(MANAGERS_PATH + ".esi")
    @patch(HELPERS_EVECORPORATION_PATH + ".esi")
    def test_user_requests_standing_for_his_alt_character(
        self, mock_esi, mock_esi_manager
    ):
        """
        given user has permission and user's alt has no standing
        when user requests standing and request is actioned by manager
        then alt has standing and user gets change notification
        """
        # setup
        self._setup_mocks(mock_esi, mock_esi_manager)
        alt_id = self.alt_character_1.character_id

        # user opens create requests page
        self.app.set_user(self.user_requestor)
        create_page_1 = self.app.get(reverse("standingsrequests:create_requests"))
        self.assertEqual(create_page_1.status_code, 200)
        create_page_2 = self.app.get(reverse("standingsrequests:request_characters"))
        self.assertEqual(create_page_2.status_code, 200)

        # user requests standing for alt
        request_standing_url = reverse(
            "standingsrequests:request_character_standing", args=[alt_id]
        )
        response = create_page_2.click(href=request_standing_url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))

        # validate new state
        self.assertTrue(StandingRequest.objects.filter(contact_id=alt_id).exists())
        my_request = StandingRequest.objects.get(contact_id=alt_id)
        self.assertFalse(my_request.is_effective)
        self.assertEqual(my_request.user, self.user_requestor)

        # manager opens manage page
        self.app.set_user(self.user_manager)
        manage_page_1 = self.app.get(reverse("standingsrequests:manage"))
        self.assertEqual(manage_page_1.status_code, 200)
        manage_page_2 = self.app.get(reverse("standingsrequests:manage_requests_list"))
        self.assertEqual(manage_page_2.status_code, 200)

        # make sure standing request is visible to manager
        data = self._parse_contacts_data(manage_page_2, "requests")
        self.assertSetEqual(set(data.keys()), {alt_id})

        # set standing in game and mark as actioned
        self._set_standing_for_alt_in_game(self.alt_character_1)
        response = self.app.put(
            reverse("standingsrequests:manage_requests_write", args=[alt_id])
        )
        self.assertEqual(response.status_code, 200)

        # validate new state
        my_request.refresh_from_db()
        self.assertEqual(my_request.action_by, self.user_manager)
        self.assertIsNotNone(my_request.action_date)
        self.assertFalse(my_request.is_effective)
        self.assertEqual(
            RequestLogEntry.objects.filter(
                action_by__user=self.user_manager,
                requested_for__character_id=alt_id,
                requested_by__user=self.user_requestor,
                request_type=RequestLogEntry.RequestType.REQUEST,
                action=RequestLogEntry.Action.CONFIRMED,
            ).count(),
            1,
        )

        # run process standing results task
        self._process_standing_requests()

        # validate final state
        my_request.refresh_from_db()
        self.assertTrue(my_request.is_effective)
        self.assertIsNotNone(my_request.effective_date)
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())

    @patch(MANAGERS_PATH + ".esi")
    @patch(HELPERS_EVECORPORATION_PATH + ".esi")
    def test_user_requests_revocation_for_his_alt_character(
        self, mock_esi, mock_esi_manager
    ):
        """
        given user's alt has standing and user has permission
        when user requests revocation and request is actioned by manager
        then alt's standing is removed and user gets change notification
        """
        # setup
        self._setup_mocks(mock_esi, mock_esi_manager)
        alt_id = self.alt_character_1.character_id
        self._set_standing_for_alt_in_game(self.alt_character_1)
        my_request = self._create_standing_for_alt(self.alt_character_1)

        # user opens create requests page
        self.app.set_user(self.user_requestor)
        create_page_1 = self.app.get(reverse("standingsrequests:create_requests"))
        self.assertEqual(create_page_1.status_code, 200)
        create_page_2 = self.app.get(reverse("standingsrequests:request_characters"))
        self.assertEqual(create_page_2.status_code, 200)

        # user requests standing for alt
        request_standing_url = reverse(
            "standingsrequests:remove_character_standing", args=[alt_id]
        )
        response = create_page_2.click(href=request_standing_url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))

        # validate new state
        self.assertTrue(my_request.is_effective)
        self.assertEqual(my_request.user, self.user_requestor)
        my_revocation = StandingRevocation.objects.get(contact_id=alt_id)
        self.assertFalse(my_revocation.is_effective)
        self.assertEqual(my_revocation.reason, StandingRevocation.Reason.OWNER_REQUEST)

        # manager opens manage page
        self.app.set_user(self.user_manager)
        manage_page_1 = self.app.get(reverse("standingsrequests:manage"))
        self.assertEqual(manage_page_1.status_code, 200)
        manage_page_2 = self.app.get(
            reverse("standingsrequests:manage_revocations_list")
        )
        self.assertEqual(manage_page_2.status_code, 200)

        # make sure standing request is visible to manager
        data = self._parse_contacts_data(manage_page_2, "revocations")
        self.assertSetEqual(set(data.keys()), {alt_id})

        # remove standing for alt in game and mark as actioned
        self._remove_standing_for_alt_in_game(self.alt_character_1)
        response = self.app.put(
            reverse("standingsrequests:manage_revocations_write", args=[alt_id])
        )
        self.assertEqual(response.status_code, 200)

        # validate new state
        my_revocation.refresh_from_db()
        self.assertEqual(my_revocation.action_by, self.user_manager)
        self.assertIsNotNone(my_revocation.action_date)
        self.assertFalse(my_revocation.is_effective)
        self.assertEqual(
            RequestLogEntry.objects.filter(
                action_by__user=self.user_manager,
                requested_for__character_id=alt_id,
                requested_by__user=self.user_requestor,
                request_type=RequestLogEntry.RequestType.REVOCATION,
                action=RequestLogEntry.Action.CONFIRMED,
                reason=StandingRevocation.Reason.OWNER_REQUEST,
            ).count(),
            1,
        )

        # run process standing results task
        self._process_standing_requests()

        # validate final state
        self.assertFalse(StandingRequest.objects.filter(contact_id=alt_id).exists())
        self.assertFalse(StandingRevocation.objects.filter(contact_id=alt_id).exists())
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())

    @patch(MANAGERS_PATH + ".esi")
    @patch(HELPERS_EVECORPORATION_PATH + ".esi")
    def test_user_requests_standing_for_his_alt_corporation(
        self, mock_esi, mock_esi_manager
    ):
        """
        given user has permission and user's alt has no standing
        and all corporation members have tokens
        when user requests standing and request is actioned by manager
        then alt has standing and user gets change notification
        """
        # setup
        self._setup_mocks(mock_esi, mock_esi_manager)
        alt_id = self.alt_corporation.corporation_id

        # user opens create requests page
        self.app.set_user(self.user_requestor)
        create_page_1 = self.app.get(reverse("standingsrequests:create_requests"))
        self.assertEqual(create_page_1.status_code, 200)
        create_page_2 = self.app.get(reverse("standingsrequests:request_corporations"))
        self.assertEqual(create_page_2.status_code, 200)

        # user requests standing for alt
        request_standing_url = reverse(
            "standingsrequests:request_corp_standing", args=[alt_id]
        )
        response = create_page_2.click(href=request_standing_url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))

        # validate new state
        self.assertTrue(StandingRequest.objects.filter(contact_id=alt_id).exists())
        my_request = StandingRequest.objects.get(contact_id=alt_id)
        self.assertFalse(my_request.is_effective)
        self.assertEqual(my_request.user, self.user_requestor)

        # manager opens manage page
        self.app.set_user(self.user_manager)
        manage_page_1 = self.app.get(reverse("standingsrequests:manage"))
        self.assertEqual(manage_page_1.status_code, 200)
        manage_page_2 = self.app.get(reverse("standingsrequests:manage_requests_list"))
        self.assertEqual(manage_page_2.status_code, 200)

        # make sure standing request is visible to manager
        data = self._parse_contacts_data(manage_page_2, "requests")
        self.assertSetEqual(set(data.keys()), {alt_id})

        # set standing in game and mark as actioned
        self._set_standing_for_alt_in_game(self.alt_corporation)
        response = self.app.put(
            reverse("standingsrequests:manage_requests_write", args=[alt_id])
        )
        self.assertEqual(response.status_code, 200)

        # validate new state
        my_request.refresh_from_db()
        self.assertEqual(my_request.action_by, self.user_manager)
        self.assertIsNotNone(my_request.action_date)
        self.assertFalse(my_request.is_effective)
        self.assertEqual(
            RequestLogEntry.objects.filter(
                action_by__user=self.user_manager,
                requested_for__corporation_id=alt_id,
                requested_by__user=self.user_requestor,
                request_type=RequestLogEntry.RequestType.REQUEST,
                action=RequestLogEntry.Action.CONFIRMED,
            ).count(),
            1,
        )

        # run process standing results task
        self._process_standing_requests()

        # validate final state
        my_request.refresh_from_db()
        self.assertTrue(my_request.is_effective)
        self.assertIsNotNone(my_request.effective_date)
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())

    @patch(MANAGERS_PATH + ".esi")
    @patch(HELPERS_EVECORPORATION_PATH + ".esi")
    def test_user_requests_revocation_for_his_alt_corporation(
        self, mock_esi, mock_esi_manager
    ):
        """
        given user's alt has standing and user has permission
        when user requests revocation and request is actioned by manager
        then alt's standing is removed and user gets change notification
        """
        # setup
        self._setup_mocks(mock_esi, mock_esi_manager)
        alt_id = self.alt_corporation.corporation_id
        self._set_standing_for_alt_in_game(self.alt_corporation)
        my_request = self._create_standing_for_alt(self.alt_corporation)

        # user opens create requests page
        self.app.set_user(self.user_requestor)
        create_page_1 = self.app.get(reverse("standingsrequests:create_requests"))
        self.assertEqual(create_page_1.status_code, 200)
        create_page_2 = self.app.get(reverse("standingsrequests:request_corporations"))
        self.assertEqual(create_page_2.status_code, 200)

        # user requests standing for alt
        request_standing_url = reverse(
            "standingsrequests:remove_corp_standing", args=[alt_id]
        )
        response = create_page_2.click(href=request_standing_url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))

        # validate new state
        self.assertTrue(my_request.is_effective)
        self.assertEqual(my_request.user, self.user_requestor)
        my_revocation = StandingRevocation.objects.get(contact_id=alt_id)
        self.assertFalse(my_revocation.is_effective)

        # manager opens manage page
        self.app.set_user(self.user_manager)
        manage_page_1 = self.app.get(reverse("standingsrequests:manage"))
        self.assertEqual(manage_page_1.status_code, 200)
        manage_page_2 = self.app.get(
            reverse("standingsrequests:manage_revocations_list")
        )
        self.assertEqual(manage_page_2.status_code, 200)

        # make sure standing request is visible to manager
        data = self._parse_contacts_data(manage_page_2, "revocations")
        self.assertSetEqual(set(data.keys()), {alt_id})

        # remove standing for alt in game and mark as actioned
        self._remove_standing_for_alt_in_game(self.alt_corporation)
        response = self.app.put(
            reverse("standingsrequests:manage_revocations_write", args=[alt_id])
        )
        self.assertEqual(response.status_code, 200)

        # validate new state
        my_revocation.refresh_from_db()
        self.assertEqual(my_revocation.action_by, self.user_manager)
        self.assertIsNotNone(my_revocation.action_date)
        self.assertFalse(my_revocation.is_effective)
        self.assertEqual(
            RequestLogEntry.objects.filter(
                action_by__user=self.user_manager,
                requested_for__corporation_id=alt_id,
                requested_by__user=self.user_requestor,
                request_type=RequestLogEntry.RequestType.REVOCATION,
                action=RequestLogEntry.Action.CONFIRMED,
            ).count(),
            1,
        )

        # run process standing results task
        self._process_standing_requests()

        # validate final state
        self.assertFalse(StandingRequest.objects.filter(contact_id=alt_id).exists())
        self.assertFalse(StandingRevocation.objects.filter(contact_id=alt_id).exists())
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())

    @patch(MANAGERS_PATH + ".esi")
    @patch(HELPERS_EVECORPORATION_PATH + ".esi")
    def test_user_requests_standing_for_his_alt_character_but_refused(
        self, mock_esi, mock_esi_manager
    ):
        """
        given user has permission and user's alt has no standing
        when user requests standing and request is refused by manager
        then request is reset, alt has no standing and user gets notification
        """
        # setup
        self._setup_mocks(mock_esi, mock_esi_manager)
        alt_id = self.alt_character_1.character_id

        # user opens create requests page
        self.app.set_user(self.user_requestor)
        create_page_1 = self.app.get(reverse("standingsrequests:create_requests"))
        self.assertEqual(create_page_1.status_code, 200)
        create_page_2 = self.app.get(reverse("standingsrequests:request_characters"))
        self.assertEqual(create_page_2.status_code, 200)

        # user requests standing for alt
        request_standing_url = reverse(
            "standingsrequests:request_character_standing", args=[alt_id]
        )
        response = create_page_2.click(href=request_standing_url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))

        # validate new state
        self.assertTrue(StandingRequest.objects.filter(contact_id=alt_id).exists())
        my_request = StandingRequest.objects.get(contact_id=alt_id)
        self.assertFalse(my_request.is_effective)
        self.assertEqual(my_request.user, self.user_requestor)

        # manager opens manage page
        self.app.set_user(self.user_manager)
        manage_page_1 = self.app.get(reverse("standingsrequests:manage"))
        self.assertEqual(manage_page_1.status_code, 200)
        manage_page_2 = self.app.get(reverse("standingsrequests:manage_requests_list"))
        self.assertEqual(manage_page_2.status_code, 200)

        # make sure standing request is visible to manager
        data = self._parse_contacts_data(manage_page_2, "requests")
        self.assertSetEqual(set(data.keys()), {alt_id})

        # Manage refused request
        response = self.app.delete(
            reverse("standingsrequests:manage_requests_write", args=[alt_id])
        )
        self.assertEqual(response.status_code, 200)

        # validate final state
        self.assertFalse(StandingRequest.objects.filter(contact_id=alt_id).exists())
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())
        self.assertEqual(
            RequestLogEntry.objects.filter(
                action_by__user=self.user_manager,
                requested_for__character_id=alt_id,
                requested_by__user=self.user_requestor,
                request_type=RequestLogEntry.RequestType.REQUEST,
                action=RequestLogEntry.Action.REJECTED,
            ).count(),
            1,
        )

    @patch(MANAGERS_PATH + ".esi")
    @patch(HELPERS_EVECORPORATION_PATH + ".esi")
    def test_user_requests_standing_for_his_alt_corporation_but_refused(
        self, mock_esi, mock_esi_manager
    ):
        """
        given user has permission and user's alt has no standing
        and all corporation members have tokens
        when user requests standing and request is actioned by manager
        then alt has standing and user gets change notification
        """
        # setup
        self._setup_mocks(mock_esi, mock_esi_manager)
        alt_id = self.alt_corporation.corporation_id

        # user opens create requests page
        self.app.set_user(self.user_requestor)
        create_page_1 = self.app.get(reverse("standingsrequests:create_requests"))
        self.assertEqual(create_page_1.status_code, 200)
        create_page_2 = self.app.get(reverse("standingsrequests:request_corporations"))
        self.assertEqual(create_page_2.status_code, 200)

        # user requests standing for alt
        request_standing_url = reverse(
            "standingsrequests:request_corp_standing", args=[alt_id]
        )
        response = create_page_2.click(href=request_standing_url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))

        # validate new state
        self.assertTrue(StandingRequest.objects.filter(contact_id=alt_id).exists())
        my_request = StandingRequest.objects.get(contact_id=alt_id)
        self.assertFalse(my_request.is_effective)
        self.assertEqual(my_request.user, self.user_requestor)

        # manager opens manage page
        self.app.set_user(self.user_manager)
        manage_page_1 = self.app.get(reverse("standingsrequests:manage"))
        self.assertEqual(manage_page_1.status_code, 200)
        manage_page_2 = self.app.get(reverse("standingsrequests:manage_requests_list"))
        self.assertEqual(manage_page_2.status_code, 200)

        # make sure standing request is visible to manager
        data = self._parse_contacts_data(manage_page_2, "requests")
        self.assertSetEqual(set(data.keys()), {alt_id})

        # Manage refused request
        response = self.app.delete(
            reverse("standingsrequests:manage_requests_write", args=[alt_id])
        )
        self.assertEqual(response.status_code, 200)

        # validate final state
        self.assertFalse(StandingRequest.objects.filter(contact_id=alt_id).exists())
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())
        self.assertEqual(
            RequestLogEntry.objects.filter(
                action_by__user=self.user_manager,
                requested_for__corporation_id=alt_id,
                requested_by__user=self.user_requestor,
                request_type=RequestLogEntry.RequestType.REQUEST,
                action=RequestLogEntry.Action.REJECTED,
            ).count(),
            1,
        )

    @patch(MANAGERS_PATH + ".esi")
    @patch(HELPERS_EVECORPORATION_PATH + ".esi")
    def test_user_requests_revocation_for_his_alt_character_but_refused(
        self, mock_esi, mock_esi_manager
    ):
        """
        given user's alt has standing and user has permission
        when user requests revocation and request is actioned by manager
        then alt's standing is removed and user gets change notification
        """
        # setup
        self._setup_mocks(mock_esi, mock_esi_manager)
        alt_id = self.alt_character_1.character_id
        self._set_standing_for_alt_in_game(self.alt_character_1)
        my_request = self._create_standing_for_alt(self.alt_character_1)

        # user opens create requests page
        self.app.set_user(self.user_requestor)
        create_page_1 = self.app.get(reverse("standingsrequests:create_requests"))
        self.assertEqual(create_page_1.status_code, 200)
        create_page_2 = self.app.get(reverse("standingsrequests:request_characters"))
        self.assertEqual(create_page_2.status_code, 200)

        # user requests standing for alt
        request_standing_url = reverse(
            "standingsrequests:remove_character_standing", args=[alt_id]
        )
        response = create_page_2.click(href=request_standing_url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))

        # validate new state
        self.assertTrue(my_request.is_effective)
        self.assertEqual(my_request.user, self.user_requestor)
        my_revocation = StandingRevocation.objects.get(contact_id=alt_id)
        self.assertFalse(my_revocation.is_effective)
        self.assertEqual(my_revocation.reason, StandingRevocation.Reason.OWNER_REQUEST)

        # manager opens manage page
        self.app.set_user(self.user_manager)
        manage_page_1 = self.app.get(reverse("standingsrequests:manage"))
        self.assertEqual(manage_page_1.status_code, 200)
        manage_page_2 = self.app.get(
            reverse("standingsrequests:manage_revocations_list")
        )
        self.assertEqual(manage_page_2.status_code, 200)

        # make sure standing request is visible to manager
        data = self._parse_contacts_data(manage_page_2, "revocations")
        self.assertSetEqual(set(data.keys()), {alt_id})

        # Manage refused request
        response = self.app.delete(
            reverse("standingsrequests:manage_revocations_write", args=[alt_id])
        )
        self.assertEqual(response.status_code, 200)

        # validate final state
        self.assertFalse(StandingRevocation.objects.filter(contact_id=alt_id).exists())
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())
        self.assertEqual(
            RequestLogEntry.objects.filter(
                action_by__user=self.user_manager,
                requested_for__character_id=alt_id,
                requested_by__user=self.user_requestor,
                request_type=RequestLogEntry.RequestType.REVOCATION,
                action=RequestLogEntry.Action.REJECTED,
                reason=StandingRevocation.Reason.OWNER_REQUEST,
            ).count(),
            1,
        )

    def test_automatic_standing_revocation_when_standing_is_reset_in_game(self):
        """
        given user's alt has standing and user has permission
        when alt's standing is reset in-game
        then alt's standing is removed and user gets change notification
        """

        # setup
        alt_id = self.alt_character_1.character_id
        self._create_standing_for_alt(self.alt_character_1)

        # run task
        self._process_standing_requests()

        # validate
        self.assertFalse(StandingRequest.objects.filter(contact_id=alt_id).exists())
        self.assertFalse(StandingRevocation.objects.filter(contact_id=alt_id).exists())
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())

    def test_automatically_create_standing_revocation_for_invalid_alts(self):
        """
        given user's alt has standing record
        when user has lost permission
        then standing revocation is automatically created
        and standing is removed after actioned by manager
        and user is notified
        """

        # setup
        alt_id = self.alt_character_1.character_id
        self._set_standing_for_alt_in_game(self.alt_character_1)
        my_request = self._create_standing_for_alt(self.alt_character_1)
        self.member_state.member_characters.remove(self.main_character_1)
        permission = AuthUtils.get_permission_by_name(
            StandingRequest.REQUEST_PERMISSION_NAME
        )
        self.user_requestor.user_permissions.remove(permission)
        self.user_requestor = User.objects.get(pk=self.user_requestor.pk)
        self.assertFalse(
            self.user_requestor.has_perm(StandingRequest.REQUEST_PERMISSION_NAME)
        )

        # run task
        tasks.validate_requests()

        # validate new state
        my_request.refresh_from_db()
        self.assertTrue(my_request.is_effective)
        my_revocation = StandingRevocation.objects.get(contact_id=alt_id)
        self.assertFalse(my_revocation.is_effective)
        self.assertEqual(
            my_revocation.reason, StandingRevocation.Reason.LOST_PERMISSION
        )

        # manager opens manage page
        self.app.set_user(self.user_manager)
        manage_page_1 = self.app.get(reverse("standingsrequests:manage"))
        self.assertEqual(manage_page_1.status_code, 200)
        manage_page_2 = self.app.get(
            reverse("standingsrequests:manage_revocations_list")
        )
        self.assertEqual(manage_page_2.status_code, 200)

        # make sure standing request is visible to manager
        data = self._parse_contacts_data(manage_page_2, "revocations")
        self.assertSetEqual(set(data.keys()), {alt_id})

        # remove standing for alt in game and mark as actioned
        self._remove_standing_for_alt_in_game(self.alt_character_1)
        response = self.app.put(
            reverse("standingsrequests:manage_revocations_write", args=[alt_id])
        )
        self.assertEqual(response.status_code, 200)

        # validate new state
        my_revocation.refresh_from_db()
        self.assertEqual(my_revocation.action_by, self.user_manager)
        self.assertIsNotNone(my_revocation.action_date)
        self.assertFalse(my_revocation.is_effective)

        # run process standing results task
        self._process_standing_requests()

        # validate final state
        self.assertFalse(StandingRequest.objects.filter(contact_id=alt_id).exists())
        self.assertFalse(StandingRevocation.objects.filter(contact_id=alt_id).exists())
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())

    @patch(TASKS_PATH + ".SR_SYNC_BLUE_ALTS_ENABLED", True)
    def test_automatically_create_standing_requests_for_valid_alts(self):
        """
        given user's alt has no standing record
        when regular standing update is run
        then standing record is automatically created for this alt
        """

        # setup
        self._set_standing_for_alt_in_game(self.alt_character_1)

        # run task
        self._process_standing_requests()

        # validate
        my_request = StandingRequest.objects.get(
            contact_id=self.alt_character_1.character_id
        )
        self.assertTrue(my_request.is_effective)
        self.assertIsNotNone(my_request.effective_date)
        self.assertEqual(
            RequestLogEntry.objects.filter(
                action_by__isnull=True,
                requested_for__character_id=self.alt_character_1.character_id,
                requested_by__user=self.user_requestor,
                request_type=RequestLogEntry.RequestType.REQUEST,
                action=RequestLogEntry.Action.CONFIRMED,
            ).count(),
            1,
        )

    @patch(TASKS_PATH + ".SR_SYNC_BLUE_ALTS_ENABLED", True)
    def test_automatically_create_standing_revocation_for_invalid_alts_2(self):
        """
        given user's alt has no standing record
        and alt has standing in game
        when user has no permission
        then standing revocation is automatically created
        and standing is removed after actioned by manager
        and user is notified
        """

        # setup
        alt_id = self.alt_character_1.character_id
        self._set_standing_for_alt_in_game(self.alt_character_1)
        self.member_state.member_characters.remove(self.main_character_1)
        permission = AuthUtils.get_permission_by_name(
            StandingRequest.REQUEST_PERMISSION_NAME
        )
        self.user_requestor.user_permissions.remove(permission)
        self.user_requestor = User.objects.get(pk=self.user_requestor.pk)
        self.assertFalse(
            self.user_requestor.has_perm(StandingRequest.REQUEST_PERMISSION_NAME)
        )

        # run tasks
        self._process_standing_requests()
        tasks.validate_requests()

        # validate new state
        req = StandingRequest.objects.get(contact_id=alt_id)
        self.assertTrue(req.is_effective)
        my_revocation = StandingRevocation.objects.get(contact_id=alt_id)
        self.assertFalse(my_revocation.is_effective)
        self.assertEqual(
            my_revocation.reason, StandingRevocation.Reason.LOST_PERMISSION
        )

        # manager opens manage page
        self.app.set_user(self.user_manager)
        manage_page_1 = self.app.get(reverse("standingsrequests:manage"))
        self.assertEqual(manage_page_1.status_code, 200)
        manage_page_2 = self.app.get(
            reverse("standingsrequests:manage_revocations_list")
        )
        self.assertEqual(manage_page_2.status_code, 200)

        # make sure standing request is visible to manager
        data = self._parse_contacts_data(manage_page_2, key="revocations")
        self.assertSetEqual(set(data.keys()), {alt_id})

        # remove standing for alt in game and mark as actioned
        self._remove_standing_for_alt_in_game(self.alt_character_1)
        response = self.app.put(
            reverse("standingsrequests:manage_revocations_write", args=[alt_id])
        )
        self.assertEqual(response.status_code, 200)

        # validate new state
        my_revocation.refresh_from_db()
        self.assertEqual(my_revocation.action_by, self.user_manager)
        self.assertIsNotNone(my_revocation.action_date)
        self.assertFalse(my_revocation.is_effective)

        # run process standing results task
        self._process_standing_requests()

        # validate final state
        self.assertFalse(StandingRequest.objects.filter(contact_id=alt_id).exists())
        self.assertFalse(StandingRevocation.objects.filter(contact_id=alt_id).exists())
        self.assertTrue(Notification.objects.filter(user=self.user_requestor).exists())



import datetime as dt

from django.test import RequestFactory
from django.urls import reverse
from django.utils.timezone import now

from allianceauth.eveonline.models import EveAllianceInfo, EveCharacter
from allianceauth.tests.auth_utils import AuthUtils
from app_utils.testing import add_character_to_user

from standingsrequests.models import CharacterAffiliation, ContactType, StandingRequest
from standingsrequests.views import standings

from ..my_test_data import (
    create_contacts_set,
    create_eve_objects,
    load_corporation_details,
    load_eve_entities,
)
from ..utils import NoSocketsTestCasePlus, json_response_to_dict_2

TEST_SCOPE = "publicData"


class TestCharacterStandingsData(NoSocketsTestCasePlus):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        load_eve_entities()
        create_eve_objects()
        cls.contact_set = create_contacts_set()
        CharacterAffiliation.objects.update_evecharacter_relations()

        member_state = AuthUtils.get_member_state()
        member_state.member_alliances.add(EveAllianceInfo.objects.get(alliance_id=3001))
        cls.user = AuthUtils.create_member("John Doe")
        cls.user = AuthUtils.add_permission_to_user_by_name(
            "standingsrequests.request_standings", cls.user
        )
        EveCharacter.objects.get(character_id=1009).delete()
        cls.main_character_1 = EveCharacter.objects.get(character_id=1002)
        cls.user_1 = AuthUtils.create_member(cls.main_character_1.character_name)
        add_character_to_user(
            cls.user_1,
            cls.main_character_1,
            is_main=True,
            scopes=[TEST_SCOPE],
        )
        cls.alt_character_1 = EveCharacter.objects.get(character_id=1007)
        add_character_to_user(
            cls.user_1,
            cls.alt_character_1,
            scopes=[TEST_SCOPE],
        )

    def test_normal_with_full_permissions(self):
        # given
        self.user = AuthUtils.add_permission_to_user_by_name(
            "standingsrequests.view", self.user
        )
        self.maxDiff = None
        request = self.factory.get(
            reverse("standingsrequests:character_standings_data")
        )
        request.user = self.user
        my_view_without_cache = standings.character_standings_data.__wrapped__
        # when
        response = my_view_without_cache(request)
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_dict_2(response, "character_id")
        expected = {1001, 1002, 1003, 1004, 1005, 1006, 1008, 1009, 1010, 1110}
        self.assertSetEqual(set(data.keys()), expected)

        data_character_1002 = data[1002]
        expected = {
            "character_id": 1002,
            "corporation_name": "Wayne Technologies",
            "alliance_name": "Wayne Enterprises",
            "faction_name": "",
            "standing": 10.0,
            "labels_str": "blue, green",
            "main_character_name": "Peter Parker",
            "state": "Member",
        }
        self.assertPartialDictEqual(data_character_1002, expected)

        data_character_1009 = data[1009]
        expected = {
            "character_id": 1009,
            "corporation_name": "Lexcorp",
            "alliance_name": "",
            "faction_name": "",
            "standing": -10.0,
            "labels_str": "red",
            "main_character_name": "-",
            "state": "-",
        }
        self.assertPartialDictEqual(data_character_1009, expected)

    def test_normal_with_basic_permission(self):
        # given
        self.maxDiff = None
        request = self.factory.get(
            reverse("standingsrequests:character_standings_data")
        )
        request.user = self.user
        my_view_without_cache = standings.character_standings_data.__wrapped__
        # when
        response = my_view_without_cache(request)
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_dict_2(response, "character_id")
        expected = {1001, 1002, 1003, 1004, 1005, 1006, 1008, 1009, 1010, 1110}
        self.assertSetEqual(set(data.keys()), expected)

        data_character_1002 = data[1002]
        expected = {
            "character_id": 1002,
            "corporation_name": "Wayne Technologies",
            "alliance_name": "Wayne Enterprises",
            "faction_name": "",
            "standing": 10.0,
            "labels_str": "blue, green",
            "main_character_name": "",
            "state": "",
        }
        self.assertPartialDictEqual(data_character_1002, expected)


class TestCorporationStandingsData(NoSocketsTestCasePlus):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        cls.contact_set = create_contacts_set()
        load_eve_entities()
        create_eve_objects()
        load_corporation_details()
        member_state = AuthUtils.get_member_state()
        member_state.member_alliances.add(EveAllianceInfo.objects.get(alliance_id=3001))
        cls.user_1 = AuthUtils.create_member("John Doe")
        cls.user_1 = AuthUtils.add_permission_to_user_by_name(
            "standingsrequests.request_standings", cls.user_1
        )
        EveCharacter.objects.get(character_id=1009).delete()
        cls.main_character_1 = EveCharacter.objects.get(character_id=1002)
        cls.user_2 = AuthUtils.create_member(cls.main_character_1.character_name)
        add_character_to_user(
            cls.user_2,
            cls.main_character_1,
            is_main=True,
            scopes=[TEST_SCOPE],
        )
        cls.alt_character_1 = EveCharacter.objects.get(character_id=1007)
        add_character_to_user(
            cls.user_2,
            cls.alt_character_1,
            scopes=[TEST_SCOPE],
        )
        StandingRequest.objects.create(
            user=cls.user_2,
            contact_id=2102,
            contact_type_id=ContactType.corporation_id,
            action_by=cls.user_1,
            action_date=now() - dt.timedelta(days=1, hours=1),
            is_effective=True,
            effective_date=now() - dt.timedelta(days=1),
        )

    def test_with_full_permissions(self):
        # given
        self.user_1 = AuthUtils.add_permission_to_user_by_name(
            "standingsrequests.view", self.user_1
        )
        self.maxDiff = None
        request = self.factory.get(
            reverse("standingsrequests:corporation_standings_data")
        )
        request.user = self.user_1
        my_view_without_cache = standings.corporation_standings_data.__wrapped__
        # when
        response = my_view_without_cache(request)
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_dict_2(response, "corporation_id")
        self.assertSetEqual(set(data.keys()), {2001, 2003, 2102})
        obj = data[2001]
        expected = {
            "corporation_id": 2001,
            "alliance_name": "Wayne Enterprises",
            "faction_name": "",
            "standing": 10.0,
            "state": "-",
            "main_character_name": "-",
        }
        self.assertPartialDictEqual(obj, expected)
        obj = data[2102]
        self.assertPartialDictEqual(
            obj,
            {
                "corporation_id": 2102,
                "alliance_name": "",
                "faction_name": "",
                "standing": -10.0,
                "state": "Member",
                "main_character_name": "Peter Parker",
            },
        )

    def test_with_basic_permissions(self):
        # given
        self.maxDiff = None
        request = self.factory.get(
            reverse("standingsrequests:corporation_standings_data")
        )
        request.user = self.user_1
        my_view_without_cache = standings.corporation_standings_data.__wrapped__
        # when
        response = my_view_without_cache(request)
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_dict_2(response, "corporation_id")
        obj = data[2102]
        self.assertPartialDictEqual(
            obj,
            {
                "corporation_id": 2102,
                "alliance_name": "",
                "faction_name": "",
                "standing": -10.0,
                "state": "",
                "main_character_name": "",
            },
        )


class TestAllianceStandingsData(NoSocketsTestCasePlus):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        cls.contact_set = create_contacts_set()
        load_eve_entities()
        create_eve_objects()
        load_corporation_details()
        member_state = AuthUtils.get_member_state()
        member_state.member_alliances.add(EveAllianceInfo.objects.get(alliance_id=3001))
        cls.user = AuthUtils.create_member("John Doe")
        cls.user = AuthUtils.add_permission_to_user_by_name(
            "standingsrequests.request_standings", cls.user
        )
        EveCharacter.objects.get(character_id=1009).delete()
        cls.main_character_1 = EveCharacter.objects.get(character_id=1002)
        cls.user_1 = AuthUtils.create_member(cls.main_character_1.character_name)
        add_character_to_user(
            cls.user_1,
            cls.main_character_1,
            is_main=True,
            scopes=[TEST_SCOPE],
        )
        cls.alt_character_1 = EveCharacter.objects.get(character_id=1007)
        add_character_to_user(
            cls.user_1,
            cls.alt_character_1,
            scopes=[TEST_SCOPE],
        )

    def test_normal(self):
        # given
        self.maxDiff = None
        request = self.factory.get(reverse("standingsrequests:alliance_standings_data"))
        request.user = self.user
        my_view_without_cache = standings.alliance_standings_data.__wrapped__
        # when
        response = my_view_without_cache(request)
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_dict_2(response, "alliance_id")
        self.assertSetEqual(set(data.keys()), {3010})
        obj = data[3010]
        self.assertPartialDictEqual(obj, {"alliance_id": 3010, "standing": -10.0})



from datetime import datetime, timedelta
from unittest.mock import patch

from django.contrib.auth.models import User
from django.test import TestCase
from django.utils.timezone import now
from eveuniverse.models import EveEntity

from allianceauth.eveonline.models import EveCharacter
from allianceauth.tests.auth_utils import AuthUtils
from app_utils.testing import (
    NoSocketsTestCase,
    _generate_token,
    _store_as_Token,
    add_character_to_user,
    add_new_token,
)

from ..core import MainOrganizations
from ..helpers.evecorporation import EveCorporation
from ..models import (
    AbstractStandingsRequest,
    CharacterAffiliation,
    Contact,
    ContactLabel,
    ContactSet,
    StandingRequest,
    StandingRevocation,
)
from .entity_type_ids import CHARACTER_BRUTOR_TYPE_ID, CHARACTER_TYPE_ID
from .my_test_data import (
    TEST_STANDINGS_ALLIANCE_ID,
    create_contacts_set,
    create_entity,
    create_standings_char,
    get_my_test_data,
    load_eve_entities,
)

CORE_PATH = "standingsrequests.core"
MODELS_PATH = "standingsrequests.models"
TEST_USER_NAME = "Peter Parker"
TEST_REQUIRED_SCOPE = "mind_reading.v1"


class TestContactSet(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.character_1001 = create_entity(EveCharacter, 1001)
        load_eve_entities()

    def test_str(self):
        my_set = ContactSet(name="My Set")
        self.assertIsInstance(str(my_set), str)

    @patch(CORE_PATH + ".STR_CORP_IDS", ["2001"])
    @patch(CORE_PATH + ".STR_ALLIANCE_IDS", [])
    def test_pilot_in_organisation_matches_corp(self):
        self.assertTrue(MainOrganizations.is_character_a_member(self.character_1001))

    @patch(CORE_PATH + ".STR_CORP_IDS", [])
    @patch(CORE_PATH + ".STR_ALLIANCE_IDS", ["3001"])
    def test_pilot_in_organisation_matches_alliance(self):
        self.assertTrue(MainOrganizations.is_character_a_member(self.character_1001))

    @patch(CORE_PATH + ".STR_CORP_IDS", [])
    @patch(CORE_PATH + ".STR_ALLIANCE_IDS", [3101])
    def test_pilot_in_organisation_doest_not_exist(self):
        self.assertFalse(MainOrganizations.is_character_a_member(self.character_1001))

    @patch(CORE_PATH + ".STR_CORP_IDS", [])
    @patch(CORE_PATH + ".STR_ALLIANCE_IDS", [])
    def test_pilot_in_organisation_matches_none(self):
        self.assertFalse(MainOrganizations.is_character_a_member(self.character_1001))


class TestContactSetCreateStanding(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.contact_set = create_contacts_set()

    def test_can_create_pilot_standing(self):
        obj = Contact.objects.create(
            contact_set=self.contact_set, eve_entity_id=1009, standing=-10
        )
        obj.labels.add(*ContactLabel.objects.all())
        self.assertIsInstance(obj, Contact)
        self.assertEqual(obj.eve_entity_id, 1009)
        self.assertEqual(obj.standing, -10)

    def test_can_create_corp_standing(self):
        obj = Contact.objects.create(
            contact_set=self.contact_set, eve_entity_id=2102, standing=-10
        )
        obj.labels.add(*ContactLabel.objects.all())
        self.assertIsInstance(obj, Contact)
        self.assertEqual(obj.eve_entity_id, 2102)
        self.assertEqual(obj.standing, -10)

    def test_can_create_alliance_standing(self):
        obj = Contact.objects.create(
            contact_set=self.contact_set, eve_entity_id=3001, standing=5
        )
        obj.labels.add(*ContactLabel.objects.all())
        self.assertIsInstance(obj, Contact)
        self.assertEqual(obj.eve_entity_id, 3001)
        self.assertEqual(obj.standing, 5)


@patch(
    MODELS_PATH + ".SR_REQUIRED_SCOPES",
    {"Member": [TEST_REQUIRED_SCOPE], "Blue": [], "": []},
)
@patch(CORE_PATH + ".STR_ALLIANCE_IDS", [TEST_STANDINGS_ALLIANCE_ID])
class TestContactSetGenerateStandingRequestsForBlueAlts(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.user = AuthUtils.create_member(TEST_USER_NAME)
        create_standings_char()
        cls.contacts_set = create_contacts_set()

    def test_should_create_new_request_for_blue_alt(self):
        # given
        alt_id = 1010
        alt = create_entity(EveCharacter, alt_id)
        add_character_to_user(self.user, alt, scopes=["dummy"])
        # when
        self.contacts_set.generate_standing_requests_for_blue_alts()
        # then
        request = StandingRequest.objects.get(contact_id=alt_id)
        self.assertTrue(request.is_effective)
        self.assertEqual(request.user, self.user)
        self.assertEqual(request.contact_id, 1010)
        self.assertEqual(request.is_effective, True)
        self.assertAlmostEqual((now() - request.request_date).seconds, 0, delta=30)
        self.assertAlmostEqual((now() - request.action_date).seconds, 0, delta=30)
        self.assertAlmostEqual((now() - request.effective_date).seconds, 0, delta=30)

    def test_should_not_create_requests_for_blue_alt_if_request_already_exists(self):
        # given
        alt_id = 1010
        alt = create_entity(EveCharacter, alt_id)
        add_character_to_user(self.user, alt, scopes=["dummy"])
        req = StandingRequest.objects.get_or_create_2(
            self.user,
            alt_id,
            StandingRequest.ContactType.CHARACTER,
        )
        # when
        self.contacts_set.generate_standing_requests_for_blue_alts()
        # then
        req.refresh_from_db()
        self.assertFalse(req.is_effective)

    def test_should_not_create_requests_for_non_blue_alts(self):
        # given
        alt_id = 1009
        alt = create_entity(EveCharacter, alt_id)
        add_character_to_user(self.user, alt, scopes=["dummy"])
        # when
        self.contacts_set.generate_standing_requests_for_blue_alts()
        # then
        self.assertFalse(StandingRequest.objects.filter(contact_id=alt_id).exists())

    def test_should_not_create_requests_for_alts_in_organization(self):
        # given
        alt_id = 1002
        main = create_entity(EveCharacter, alt_id)
        add_character_to_user(self.user, main, is_main=True, scopes=["dummy"])
        # when
        self.contacts_set.generate_standing_requests_for_blue_alts()
        # then
        self.assertFalse(StandingRequest.objects.filter(contact_id=alt_id).exists())


class TestAbstractStandingsRequest(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        ContactSet.objects.all().delete()
        create_contacts_set()
        cls.user_requestor = User.objects.create_user(
            "Roger Requestor", "rr@example.com", "password"
        )

    def test_should_say_standing_request(self):
        # given
        my_request = StandingRequest(
            user=self.user_requestor,
            contact_id=1002,
            contact_type_id=CHARACTER_BRUTOR_TYPE_ID,
        )
        # then
        self.assertTrue(my_request.is_standing_request)
        self.assertFalse(my_request.is_standing_revocation)

    def test_should_say_standing_revocation(self):
        # given
        my_request = StandingRevocation(
            user=self.user_requestor,
            contact_id=1002,
            contact_type_id=CHARACTER_BRUTOR_TYPE_ID,
        )
        # then
        self.assertFalse(my_request.is_standing_request)
        self.assertTrue(my_request.is_standing_revocation)


class TestStandingRequest(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        ContactSet.objects.all().delete()
        create_contacts_set()
        cls.user_manager = User.objects.create_user(
            "Mike Manager", "mm@example.com", "password"
        )
        cls.user_requestor = User.objects.create_user(
            "Roger Requestor", "rr@example.com", "password"
        )

    def test_is_standing_satisfied(self):
        class MyStandingRequest(AbstractStandingsRequest):
            EXPECT_STANDING_LTEQ = 5.0
            EXPECT_STANDING_GTEQ = 0.0

        self.assertTrue(MyStandingRequest.is_standing_satisfied(5))
        self.assertTrue(MyStandingRequest.is_standing_satisfied(0))
        self.assertFalse(MyStandingRequest.is_standing_satisfied(-10))
        self.assertFalse(MyStandingRequest.is_standing_satisfied(10))
        self.assertFalse(MyStandingRequest.is_standing_satisfied(None))

    def test_check_standing_satisfied_check_only(self):
        my_request = StandingRequest(
            user=self.user_requestor, contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
        )
        self.assertTrue(my_request.evaluate_effective_standing(check_only=True))

        my_request = StandingRequest(
            user=self.user_requestor,
            contact_id=1002,
            contact_type_id=CHARACTER_BRUTOR_TYPE_ID,
        )
        self.assertTrue(my_request.evaluate_effective_standing(check_only=True))

        my_request = StandingRequest(
            user=self.user_requestor,
            contact_id=1003,
            contact_type_id=CHARACTER_BRUTOR_TYPE_ID,
        )
        self.assertTrue(my_request.evaluate_effective_standing(check_only=True))

        my_request = StandingRequest(
            user=self.user_requestor,
            contact_id=1005,
            contact_type_id=CHARACTER_BRUTOR_TYPE_ID,
        )
        self.assertFalse(my_request.evaluate_effective_standing(check_only=True))

        my_request = StandingRequest(
            user=self.user_requestor,
            contact_id=1009,
            contact_type_id=CHARACTER_BRUTOR_TYPE_ID,
        )
        self.assertFalse(my_request.evaluate_effective_standing(check_only=True))

    def test_check_standing_satisfied_no_standing(self):
        my_request = StandingRequest.objects.create(
            user=self.user_requestor, contact_id=1999, contact_type_id=CHARACTER_TYPE_ID
        )
        self.assertFalse(my_request.evaluate_effective_standing(check_only=True))

    def test_mark_standing_effective_1(self):
        # given
        my_request = StandingRequest.objects.create(
            user=self.user_requestor, contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
        )
        # when
        my_request.mark_effective()
        # then
        my_request.refresh_from_db()
        self.assertTrue(my_request.is_effective)
        self.assertIsInstance(my_request.effective_date, datetime)

    def test_mark_standing_effective_2(self):
        # given
        my_request = StandingRequest.objects.create(
            user=self.user_requestor, contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
        )
        my_date = now() - timedelta(days=5, hours=4)
        # when
        my_request.mark_effective(date=my_date)
        # then
        my_request.refresh_from_db()
        self.assertTrue(my_request.is_effective)
        self.assertEqual(my_request.effective_date, my_date)

    def test_check_standing_satisfied_and_mark(self):
        my_request = StandingRequest.objects.create(
            user=self.user_requestor, contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
        )
        self.assertTrue(my_request.evaluate_effective_standing())
        my_request.refresh_from_db()
        self.assertTrue(my_request.is_effective)
        self.assertIsInstance(my_request.effective_date, datetime)

    def test_mark_standing_actioned(self):
        # given
        my_request = StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
        )
        # when
        my_request.mark_actioned(self.user_manager)
        # then
        my_request.refresh_from_db()
        self.assertEqual(my_request.action_by, self.user_manager)
        self.assertIsInstance(my_request.action_date, datetime)
        self.assertEqual(my_request.reason, StandingRequest.Reason.NONE)

    def test_mark_standing_actioned_with_reason(self):
        # given
        my_request = StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
        )
        # when
        my_request.mark_actioned(
            user=self.user_manager, reason=StandingRequest.Reason.STANDING_IN_GAME
        )
        # then
        my_request.refresh_from_db()
        self.assertEqual(my_request.action_by, self.user_manager)
        self.assertIsInstance(my_request.action_date, datetime)
        self.assertEqual(my_request.reason, StandingRequest.Reason.STANDING_IN_GAME)

    def test_check_standing_actioned_timeout_already_effective(self):
        my_request = StandingRequest(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            action_by=self.user_manager,
            action_date=now(),
            is_effective=True,
        )
        self.assertIsNone(my_request.check_actioned_timeout())

    def test_check_standing_actioned_timeout_not_actioned(self):
        my_request = StandingRequest(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            is_effective=False,
        )
        self.assertIsNone(my_request.check_actioned_timeout())

    def test_check_standing_actioned_timeout_no_contact_set(self):
        ContactSet.objects.all().delete()
        my_request = StandingRequest(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            action_by=self.user_manager,
            action_date=now(),
            is_effective=False,
        )
        self.assertIsNone(my_request.check_actioned_timeout())

    def test_check_standing_actioned_timeout_after_deadline(self):
        my_request = StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            action_by=self.user_manager,
            action_date=now() - timedelta(hours=25),
            is_effective=False,
        )
        self.assertEqual(my_request.check_actioned_timeout(), self.user_manager)
        my_request.refresh_from_db()
        self.assertIsNone(my_request.action_by)
        self.assertIsNone(my_request.action_date)

    def test_check_standing_actioned_timeout_before_deadline(self):
        my_request = StandingRequest(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            action_by=self.user_manager,
            action_date=now(),
            is_effective=False,
        )
        self.assertFalse(my_request.check_actioned_timeout())

    def test_reset_to_initial(self):
        my_request = StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            action_by=self.user_manager,
            action_date=now(),
            is_effective=True,
            effective_date=now(),
        )
        my_request.reset_to_initial()
        my_request.refresh_from_db()
        self.assertFalse(my_request.is_effective)
        self.assertIsNone(my_request.effective_date)
        self.assertIsNone(my_request.action_by)
        self.assertIsNone(my_request.action_date)

    def test_delete_for_non_effective_dont_add_revocation(self):
        my_request_effective = StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            is_effective=False,
        )
        my_request_effective.delete()
        self.assertFalse(
            StandingRequest.objects.filter(
                contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
            ).exists()
        )
        self.assertFalse(
            StandingRevocation.objects.filter(
                contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
            ).exists()
        )

    def test_delete_for_effective_add_revocation(self):
        my_request_effective = StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            action_by=self.user_manager,
            action_date=now(),
            is_effective=True,
            effective_date=now(),
        )
        my_request_effective.delete()
        self.assertFalse(
            StandingRequest.objects.filter(
                contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
            ).exists()
        )
        self.assertTrue(
            StandingRevocation.objects.filter(
                contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
            ).exists()
        )

    def test_delete_for_pending_add_revocation(self):
        my_request_effective = StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            action_by=self.user_manager,
            action_date=now(),
            is_effective=False,
        )
        my_request_effective.delete()
        self.assertFalse(
            StandingRequest.objects.filter(
                contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
            ).exists()
        )
        self.assertTrue(
            StandingRevocation.objects.filter(
                contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
            ).exists()
        )

    def test_delete_for_effective_dont_add_another_revocation(self):
        my_request_effective = StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=1001,
            contact_type_id=CHARACTER_TYPE_ID,
            action_by=self.user_manager,
            action_date=now(),
            is_effective=True,
            effective_date=now(),
        )
        StandingRevocation.objects.add_revocation(
            1001, StandingRevocation.ContactType.CHARACTER
        )
        my_request_effective.delete()
        self.assertFalse(
            StandingRequest.objects.filter(
                contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
            ).exists()
        )
        self.assertEqual(
            StandingRevocation.objects.filter(
                contact_id=1001, contact_type_id=CHARACTER_TYPE_ID
            ).count(),
            1,
        )


class TestStandingRequestClassMethods(NoSocketsTestCase):
    @patch(MODELS_PATH + ".SR_REQUIRED_SCOPES", {"Guest": ["publicData"]})
    @patch(MODELS_PATH + ".EveCorporation.get_by_id")
    def test_can_request_corporation_standing_good(self, mock_get_corp_by_id):
        """user has tokens for all 3 chars of corp"""
        mock_get_corp_by_id.return_value = EveCorporation(
            **get_my_test_data()["EveCorporationInfo"]["2001"]
        )
        my_user = AuthUtils.create_user("John Doe")
        for character_id, character in get_my_test_data()["EveCharacter"].items():
            if character["corporation_id"] == 2001:
                my_character = EveCharacter.objects.create(**character)
                _store_as_Token(
                    _generate_token(
                        character_id=my_character.character_id,
                        character_name=my_character.character_name,
                        scopes=["publicData"],
                    ),
                    my_user,
                )

        self.assertTrue(StandingRequest.can_request_corporation_standing(2001, my_user))

    @patch(MODELS_PATH + ".SR_REQUIRED_SCOPES", {"Guest": ["publicData"]})
    @patch(MODELS_PATH + ".EveCorporation.get_by_id")
    def test_can_request_corporation_standing_incomplete(self, mock_get_corp_by_id):
        """user has tokens for only 2 / 3 chars of corp"""
        mock_get_corp_by_id.return_value = EveCorporation(
            **get_my_test_data()["EveCorporationInfo"]["2001"]
        )
        my_user = AuthUtils.create_user("John Doe")
        for character_id, character in get_my_test_data()["EveCharacter"].items():
            if character_id in [1001, 1002]:
                my_character = EveCharacter.objects.create(**character)
                _store_as_Token(
                    _generate_token(
                        character_id=my_character.character_id,
                        character_name=my_character.character_name,
                        scopes=["publicData"],
                    ),
                    my_user,
                )

        self.assertFalse(
            StandingRequest.can_request_corporation_standing(2001, my_user)
        )

    @patch(
        MODELS_PATH + ".SR_REQUIRED_SCOPES",
        {"Guest": ["publicData", "esi-mail.read_mail.v1"]},
    )
    @patch(MODELS_PATH + ".EveCorporation.get_by_id")
    def test_can_request_corporation_standing_wrong_scope(self, mock_get_corp_by_id):
        """user has tokens for only 3 / 3 chars of corp, but wrong scopes"""
        mock_get_corp_by_id.return_value = EveCorporation(
            **(get_my_test_data()["EveCorporationInfo"]["2001"])
        )
        my_user = AuthUtils.create_user("John Doe")
        for character_id, character in get_my_test_data()["EveCharacter"].items():
            if character_id in [1001, 1002]:
                my_character = EveCharacter.objects.create(**character)
                _store_as_Token(
                    _generate_token(
                        character_id=my_character.character_id,
                        character_name=my_character.character_name,
                        scopes=["publicData"],
                    ),
                    my_user,
                )

        self.assertFalse(
            StandingRequest.can_request_corporation_standing(2001, my_user)
        )

    @patch(MODELS_PATH + ".SR_REQUIRED_SCOPES", {"Guest": ["publicData"]})
    @patch(MODELS_PATH + ".EveCorporation.get_by_id")
    def test_can_request_corporation_standing_good_another_user(
        self, mock_get_corp_by_id
    ):
        """there are tokens for all 3 chars of corp, but for another user"""
        mock_get_corp_by_id.return_value = EveCorporation(
            **get_my_test_data()["EveCorporationInfo"]["2001"]
        )
        user_1 = AuthUtils.create_user("John Doe")
        for character_id, character in get_my_test_data()["EveCharacter"].items():
            if character["corporation_id"] == 2001:
                my_character = EveCharacter.objects.create(**character)
                add_character_to_user(
                    user_1,
                    my_character,
                    scopes=["publicData"],
                )

        user_2 = AuthUtils.create_user("Mike Myers")
        self.assertFalse(StandingRequest.can_request_corporation_standing(2001, user_2))


class TestStandingRequestGetRequiredScopesForState(NoSocketsTestCase):
    @patch(MODELS_PATH + ".SR_REQUIRED_SCOPES", {"member": ["abc"]})
    def test_return_scopes_if_defined_for_state(self):
        expected = ["abc"]
        self.assertListEqual(
            StandingRequest.get_required_scopes_for_state("member"), expected
        )

    @patch(MODELS_PATH + ".SR_REQUIRED_SCOPES", {"member": ["abc"]})
    def test_return_empty_list_if_not_defined_for_state(self):
        expected = []
        self.assertListEqual(
            StandingRequest.get_required_scopes_for_state("guest"), expected
        )

    @patch(MODELS_PATH + ".SR_REQUIRED_SCOPES", {"member": ["abc"]})
    def test_return_empty_list_if_state_is_note(self):
        expected = []
        self.assertListEqual(
            StandingRequest.get_required_scopes_for_state(None), expected
        )


@patch(MODELS_PATH + ".StandingRequest.get_required_scopes_for_state")
class TestStandingsManagerHasRequiredScopesForRequest(NoSocketsTestCase):
    def test_true_when_user_has_required_scopes(
        self, mock_get_required_scopes_for_state
    ):
        mock_get_required_scopes_for_state.return_value = ["abc"]
        user = AuthUtils.create_member("Bruce Wayne")
        character = AuthUtils.add_main_character_2(
            user=user,
            name="Batman",
            character_id=2099,
            corp_id=2001,
            corp_name="Wayne Tech",
        )
        add_new_token(user, character, ["abc"])
        self.assertTrue(StandingRequest.has_required_scopes_for_request(character))

    def test_false_when_user_does_not_have_required_scopes(
        self, mock_get_required_scopes_for_state
    ):
        mock_get_required_scopes_for_state.return_value = ["xyz"]
        user = AuthUtils.create_member("Bruce Wayne")
        character = AuthUtils.add_main_character_2(
            user=user,
            name="Batman",
            character_id=2099,
            corp_id=2001,
            corp_name="Wayne Tech",
        )
        add_new_token(user, character, ["abc"])
        self.assertFalse(StandingRequest.has_required_scopes_for_request(character))

    def test_false_when_user_state_can_not_be_determinded(
        self, mock_get_required_scopes_for_state
    ):
        mock_get_required_scopes_for_state.return_value = ["abc"]
        character = create_entity(EveCharacter, 1002)
        self.assertFalse(StandingRequest.has_required_scopes_for_request(character))


class TestCharacterAffiliation(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_eve_entities()

    def test_get_character_name_exists(self):
        my_assoc = CharacterAffiliation.objects.create(
            character_id=1002, corporation_id=2001
        )
        self.assertEqual(my_assoc.character_name, "Peter Parker")

    def test_get_character_name_not_exists(self):
        character = EveEntity.objects.create(id=1999)
        my_assoc = CharacterAffiliation.objects.create(
            character=character, corporation_id=2001
        )
        self.assertIsNone(my_assoc.character_name)



from allianceauth.eveonline.models import EveCharacter
from app_utils.testing import NoSocketsTestCase

from ..helpers.evecharacter import EveCharacterHelper
from .my_test_data import (
    create_contacts_set,
    create_entity,
    generate_eve_entities_from_allianceauth,
    get_my_test_data,
)

MODULE_PATH = "standingsrequests.helpers.evecorporation"


class TestEveCharacter(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        create_contacts_set()
        create_entity(EveCharacter, 1001)
        generate_eve_entities_from_allianceauth()

    def test_init_with_data_has_alliance(self):
        character = EveCharacterHelper(character_id=1002)
        self.assertEqual(character.character_id, 1002)
        self.assertEqual(character.character_name, "Peter Parker")
        self.assertEqual(character.corporation_id, 2001)
        self.assertEqual(character.corporation_name, "Wayne Technologies")
        self.assertEqual(character.alliance_id, 3001)
        self.assertEqual(character.alliance_name, "Wayne Enterprises")

    def test_init_with_data_has_no_alliance(self):
        character = EveCharacterHelper(character_id=1004)
        self.assertEqual(character.character_id, 1004)
        self.assertEqual(character.character_name, "Kara Danvers")
        self.assertEqual(character.corporation_id, 2003)
        self.assertEqual(character.corporation_name, "CatCo Worldwide Media")
        self.assertIsNone(character.alliance_id)
        self.assertIsNone(character.alliance_name)

    def test_init_with_data_has_no_main(self):
        character = EveCharacterHelper(character_id=1001)
        self.assertEqual(character.character_id, 1001)
        self.assertEqual(character.character_name, "Bruce Wayne")
        self.assertEqual(character.corporation_id, 2001)
        self.assertEqual(character.corporation_name, "Wayne Technologies")
        self.assertEqual(character.alliance_id, 3001)
        self.assertEqual(character.alliance_name, "Wayne Enterprises")

    def test_init_without_data(self):
        EveCharacter.objects.filter(character_id=1001).delete()
        data = get_my_test_data()["EveCharacter"]["1001"]
        EveCharacter.objects.create(**data)
        character = EveCharacterHelper(character_id=1001)
        self.assertEqual(character.character_id, 1001)
        self.assertEqual(character.character_name, "Bruce Wayne")



from unittest.mock import Mock, patch

from django.contrib.sessions.middleware import SessionMiddleware
from django.http import Http404
from django.test import RequestFactory, override_settings
from django.urls import reverse
from esi.models import Token

from allianceauth.eveonline.models import EveCharacter
from allianceauth.tests.auth_utils import AuthUtils
from app_utils.testing import (
    NoSocketsTestCase,
    add_character_to_user,
    create_user_from_evecharacter,
)

from standingsrequests.core import ContactType
from standingsrequests.helpers.evecorporation import EveCorporation
from standingsrequests.models import (
    RequestLogEntry,
    StandingRequest,
    StandingRevocation,
)
from standingsrequests.views import create_requests

from ..my_test_data import (
    TEST_STANDINGS_API_CHARID,
    TEST_STANDINGS_API_CHARNAME,
    TestViewPagesBase,
    create_contacts_set,
    create_entity,
    create_standings_char,
    esi_get_corporations_corporation_id,
    esi_post_characters_affiliation,
    esi_post_universe_names,
    get_my_test_data,
    load_eve_entities,
)

CORE_PATH = "standingsrequests.core"
MODELS_PATH = "standingsrequests.models"
MANAGERS_PATH = "standingsrequests.managers"
HELPERS_EVECORPORATION_PATH = "standingsrequests.helpers.evecorporation"
VIEWS_PATH = "standingsrequests.views.create_requests"


@patch(CORE_PATH + ".STANDINGS_API_CHARID", TEST_STANDINGS_API_CHARID)
@patch(VIEWS_PATH + ".update_all")
@patch(VIEWS_PATH + ".messages")
class TestViewAuthPage(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        load_eve_entities()
        cls.owner_character = create_standings_char()

    def make_request(self, user, character):
        token = Mock(spec=Token)
        token.character_id = character.character_id
        request = self.factory.get(reverse("standingsrequests:view_auth_page"))
        request.user = user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        orig_view = create_requests.view_auth_page.__wrapped__.__wrapped__.__wrapped__
        return orig_view(request, token)

    @patch(CORE_PATH + ".SR_OPERATION_MODE", "corporation")
    def test_for_corp_when_provided_standingschar_return_success(
        self, mock_messages, mock_update_all
    ):
        # given
        user = AuthUtils.create_user(TEST_STANDINGS_API_CHARNAME)
        add_character_to_user(user, self.owner_character, is_main=True)
        # when
        response = self.make_request(user, self.owner_character)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:index"))
        self.assertTrue(mock_messages.success.called)
        self.assertFalse(mock_messages.error.called)
        self.assertTrue(mock_update_all.delay.called)

    @patch(CORE_PATH + ".SR_OPERATION_MODE", "corporation")
    def test_when_not_provided_standingschar_return_error(
        self, mock_messages, mock_update_all
    ):
        create_standings_char()
        user = AuthUtils.create_user("Clark Kent")
        character = AuthUtils.add_main_character_2(user, user.username, 1002)
        response = self.make_request(user, character)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:index"))
        self.assertFalse(mock_messages.success.called)
        self.assertTrue(mock_messages.error.called)
        self.assertFalse(mock_update_all.delay.called)

    @patch(CORE_PATH + ".SR_OPERATION_MODE", "alliance")
    def test_for_alliance_when_provided_standingschar_return_success(
        self, mock_messages, mock_update_all
    ):
        user = AuthUtils.create_user(TEST_STANDINGS_API_CHARNAME)
        response = self.make_request(user, self.owner_character)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:index"))
        self.assertTrue(mock_messages.success.called)
        self.assertFalse(mock_messages.error.called)
        self.assertTrue(mock_update_all.delay.called)

    @patch(CORE_PATH + ".SR_OPERATION_MODE", "alliance")
    def test_for_alliance_when_provided_standingschar_not_in_alliance_return_error(
        self, mock_messages, mock_update_all
    ):
        user = AuthUtils.create_user(TEST_STANDINGS_API_CHARNAME)
        character = create_entity(EveCharacter, 1007)
        add_character_to_user(user, character)
        response = self.make_request(user, character)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:index"))
        self.assertFalse(mock_messages.success.called)
        self.assertTrue(mock_messages.error.called)
        self.assertFalse(mock_update_all.delay.called)


@patch(
    "allianceauth.notifications.templatetags.auth_notifications.Notification.objects.user_unread_count",
    lambda *args, **kwargs: 1,
)
@patch(CORE_PATH + ".STANDINGS_API_CHARID", TEST_STANDINGS_API_CHARID)
@patch(MANAGERS_PATH + ".SR_NOTIFICATIONS_ENABLED", True)
@patch(HELPERS_EVECORPORATION_PATH + ".esi")
class TestViewsBasics(TestViewPagesBase):
    def _setup_mocks(self, mock_esi):
        mock_Corporation = mock_esi.client.Corporation
        mock_Corporation.get_corporations_corporation_id.side_effect = (
            esi_get_corporations_corporation_id
        )
        mock_esi.client.Universe.post_universe_names.side_effect = (
            esi_post_universe_names
        )

    def test_should_redirect_to_create_requests_page_for_requestor_1(self, mock_esi):
        # given
        request = self.factory.get(reverse("standingsrequests:index"))
        request.user = self.user_requestor
        # when
        response = create_requests.index_view(request)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))

    def test_should_redirect_to_create_requests_page_for_requestor_2(self, mock_esi):
        # given
        request = self.factory.get(reverse("standingsrequests:index"))
        request.user = self.user_requestor
        StandingRequest.objects.get_or_create_2(
            self.user_requestor,
            self.alt_character_1.character_id,
            StandingRequest.ContactType.CHARACTER,
        )
        # when
        response = create_requests.index_view(request)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))

    def test_should_redirect_to_create_requests_page_for_manger(self, mock_esi):
        # given
        request = self.factory.get(reverse("standingsrequests:index"))
        request.user = self.user_requestor
        # when
        response = create_requests.index_view(request)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))


#     def test_should_redirect_to_manage_requests_page_1(self, mock_esi):
#         # given
#         request = self.factory.get(reverse("standingsrequests:index"))
#         request.user = self.user_manager
#         StandingRequest.objects.get_or_create_2(
#             self.user_requestor,
#             self.alt_character_1.character_id,
#             StandingRequest.ContactType.CHARACTER,
#         )
#         # when
#         response = create_requests.index_view(request)
#         # then
#         self.assertEqual(response.status_code, 302)
#         self.assertEqual(response.url, reverse("standingsrequests:manage"))

#     def test_should_redirect_to_manage_requests_page_2(self, mock_esi):
#         # given
#         request = self.factory.get(reverse("standingsrequests:index"))
#         request.user = self.user_manager
#         self._create_standing_for_alt(self.alt_character_1)
#         StandingRevocation.objects.add_revocation(
#             self.alt_character_1.character_id,
#             StandingRevocation.ContactType.CHARACTER,
#             user=self.user_requestor,
#         )
#         # when
#         response = create_requests.index_view(request)
#         # then
#         self.assertEqual(response.status_code, 302)
#         self.assertEqual(response.url, reverse("standingsrequests:manage"))

#     def test_user_can_open_create_requests_page(self, mock_esi):
#         request = self.factory.get(reverse("standingsrequests:create_requests"))
#         request.user = self.user_requestor
#         response = create_requests.create_requests(request)
#         self.assertEqual(response.status_code, 200)

#     def test_user_can_open_pilots_standing(self, mock_esi):
#         request = self.factory.get(reverse("standingsrequests:view_pilots"))
#         request.user = self.user_manager
#         response = create_requests.view_pilots_standings(request)
#         self.assertEqual(response.status_code, 200)

#     def test_user_can_open_groups_standing(self, mock_esi):
#         request = self.factory.get(reverse("standingsrequests:view_groups"))
#         request.user = self.user_manager
#         response = create_requests.view_groups_standings(request)
#         self.assertEqual(response.status_code, 200)

#     def test_user_can_open_manage_requests(self, mock_esi):
#         request = self.factory.get(reverse("standingsrequests:manage"))
#         request.user = self.user_manager
#         response = create_requests.manage_standings(request)
#         self.assertEqual(response.status_code, 200)

#     def test_user_can_open_accepted_requests(self, mock_esi):
#         request = self.factory.get(reverse("standingsrequests:effective_requests"))
#         request.user = self.user_manager
#         response = create_requests.effective_requests(request)
#         self.assertEqual(response.status_code, 200)


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
@patch(MODELS_PATH + ".SR_REQUIRED_SCOPES", {"Guest": ["publicData"]})
class TestRequestCharacterStanding(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        create_contacts_set()
        create_entity(EveCharacter, 1001)
        cls.user, _ = create_user_from_evecharacter(
            1001, permissions=["standingsrequests.request_standings"]
        )

    def _view_request_pilot_standing(self, character_id: int) -> bool:
        request = self.factory.get(
            reverse(
                "standingsrequests:request_character_standing",
                args=[character_id],
            )
        )
        request.user = self.user
        with patch(VIEWS_PATH + ".messages.error") as mock_message, patch(
            MANAGERS_PATH + ".esi"
        ) as mock_esi:
            mock_esi.client.Character.post_characters_affiliation.side_effect = (
                esi_post_characters_affiliation
            )
            mock_esi.client.Corporation.get_corporations_corporation_id.side_effect = (
                esi_get_corporations_corporation_id
            )
            mock_esi.client.Universe.post_universe_names.side_effect = (
                esi_post_universe_names
            )
            response = create_requests.request_character_standing(request, character_id)
            success = not mock_message.called
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))
        return success

    def test_should_create_new_request(self):
        # when
        alt_character = create_entity(EveCharacter, 1008)
        add_character_to_user(self.user, alt_character, scopes=["publicData"])
        result = self._view_request_pilot_standing(alt_character.character_id)
        # then
        self.assertTrue(result)
        obj = StandingRequest.objects.get(contact_id=alt_character.character_id)
        self.assertFalse(obj.is_actioned)
        self.assertFalse(obj.is_effective)

    def test_should_not_create_new_request_if_character_has_pending_request(self):
        # given
        alt_character = create_entity(EveCharacter, 1110)
        add_character_to_user(self.user, alt_character, scopes=["publicData"])
        StandingRequest.objects.create(
            contact_id=alt_character.character_id,
            contact_type_id=ContactType.character_id,
            user=self.user,
        )
        # when
        result = self._view_request_pilot_standing(alt_character.character_id)
        # then
        self.assertFalse(result)

    def test_should_not_create_new_request_if_character_has_pending_revocation(self):
        # given
        alt_character = create_entity(EveCharacter, 1110)
        add_character_to_user(self.user, alt_character, scopes=["publicData"])
        StandingRevocation.objects.create(
            contact_id=alt_character.character_id,
            contact_type_id=ContactType.character_id,
            user=self.user,
        )
        # when
        result = self._view_request_pilot_standing(alt_character.character_id)
        # then
        self.assertFalse(result)

    def test_should_not_create_new_request_if_character_is_missing_scopes(self):
        # given
        alt_character = create_entity(EveCharacter, 1009)
        add_character_to_user(self.user, alt_character)
        # when
        result = self._view_request_pilot_standing(alt_character.character_id)
        # then
        self.assertFalse(result)

    def test_should_not_create_new_request_if_character_is_not_owned_by_anyone(self):
        # given
        random_character = create_entity(EveCharacter, 1007)
        # when
        result = self._view_request_pilot_standing(random_character.character_id)
        # then
        self.assertFalse(result)

    def test_should_not_create_new_request_if_character_is_owned_by_sombody_else(self):
        # given
        user = AuthUtils.create_member("Peter Parker")
        other_character = create_entity(EveCharacter, 1006)
        add_character_to_user(user, other_character, scopes=["publicData"])
        # when
        result = self._view_request_pilot_standing(other_character.character_id)
        # then
        self.assertFalse(result)

    def test_should_auto_confirm_new_request_if_standing_is_satisfied(self):
        # given
        alt_character = create_entity(EveCharacter, 1110)
        add_character_to_user(self.user, alt_character, scopes=["publicData"])
        # when
        result = self._view_request_pilot_standing(alt_character.character_id)
        # then
        self.assertTrue(result)
        obj = StandingRequest.objects.get(contact_id=alt_character.character_id)
        self.assertTrue(obj.is_effective)
        self.assertEqual(
            RequestLogEntry.objects.filter(
                action_by__isnull=True,
                requested_for__character_id=alt_character.character_id,
                requested_by__user=self.user,
                request_type=RequestLogEntry.RequestType.REQUEST,
                action=RequestLogEntry.Action.CONFIRMED,
                reason=StandingRequest.Reason.STANDING_IN_GAME,
            ).count(),
            1,
        )


@patch(CORE_PATH + ".STR_ALLIANCE_IDS", [3001])
@patch(CORE_PATH + ".SR_OPERATION_MODE", "alliance")
class TestRemoveCharacterStanding(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        create_contacts_set()
        create_entity(EveCharacter, 1001)
        cls.user, _ = create_user_from_evecharacter(
            1001, permissions=["standingsrequests.request_standings"]
        )

    def _view_request_pilot_standing(self, character_id: int) -> bool:
        request = self.factory.get(
            reverse("standingsrequests:remove_character_standing", args=[character_id])
        )
        request.user = self.user
        with patch(VIEWS_PATH + ".messages.warning") as mock_message:
            response = create_requests.remove_character_standing(request, character_id)
            success = not mock_message.called
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))
        return success

    def test_should_remove_valid_request(self):
        # given
        alt_character = create_entity(EveCharacter, 1110)
        add_character_to_user(self.user, alt_character, scopes=["publicData"])
        StandingRequest.objects.get_or_create_2(
            user=self.user,
            contact_id=alt_character.character_id,
            contact_type=StandingRequest.ContactType.CHARACTER,
        )
        # when
        result = self._view_request_pilot_standing(alt_character.character_id)
        # then
        self.assertTrue(result)
        self.assertFalse(
            StandingRequest.objects.filter(
                contact_id=alt_character.character_id
            ).exists()
        )

    def test_should_not_remove_request_if_character_not_owned_by_anyone(self):
        # given
        random_character = create_entity(EveCharacter, 1007)
        # when
        with self.assertRaises(Http404):
            self._view_request_pilot_standing(random_character.character_id)

    def test_should_not_remove_request_if_character_is_owned_by_sombody_else(self):
        # given
        user = AuthUtils.create_member("Peter Parker")
        other_character = create_entity(EveCharacter, 1006)
        add_character_to_user(user, other_character, scopes=["publicData"])
        # when
        with self.assertRaises(Http404):
            self._view_request_pilot_standing(other_character.character_id)

    def test_should_return_false_if_character_in_organization(self):
        # given
        alt_character = create_entity(EveCharacter, 1002)
        add_character_to_user(self.user, alt_character, scopes=["publicData"])
        # when
        with self.assertRaises(Http404):
            self._view_request_pilot_standing(alt_character.character_id)

    # I believe we do not need this requirement
    # def test_should_create_revocation_if_character_has_satisfied_standing(self):
    #     # given
    #     alt_character = create_entity(EveCharacter, 1110)
    #     add_character_to_user(self.user, alt_character, scopes=["publicData"])
    #     # when
    #     result = self._view_request_pilot_standing(alt_character.character_id)
    #     # then
    #     self.assertTrue(result)

    def test_should_return_false_if_character_has_no_standing_request(self):
        # given
        alt_character = create_entity(EveCharacter, 1008)
        add_character_to_user(self.user, alt_character, scopes=["publicData"])
        # when
        with self.assertRaises(Http404):
            self._view_request_pilot_standing(alt_character.character_id)


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
@patch(MODELS_PATH + ".SR_REQUIRED_SCOPES", {"Guest": ["publicData"]})
class TestRequestCorporationStanding(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        create_contacts_set()
        create_entity(EveCharacter, 1001)
        cls.user, _ = create_user_from_evecharacter(
            1001, permissions=["standingsrequests.request_standings"]
        )

    def _view_request_corp_standing(self, corporation_id: int) -> bool:
        request = self.factory.get(
            reverse("standingsrequests:request_corp_standing", args=[corporation_id])
        )
        request.user = self.user
        with patch(MODELS_PATH + ".EveCorporation.get_by_id") as mock_get_corp_by_id:
            mock_get_corp_by_id.return_value = EveCorporation(
                **get_my_test_data()["EveCorporationInfo"]["2102"]
            )
            with patch(VIEWS_PATH + ".messages.warning") as mock_message, patch(
                MANAGERS_PATH + ".esi"
            ) as mock_esi:
                mock_esi.client.Character.post_characters_affiliation.side_effect = (
                    esi_post_characters_affiliation
                )
                mock_Corporation = mock_esi.client.Corporation
                mock_Corporation.get_corporations_corporation_id.side_effect = (
                    esi_get_corporations_corporation_id
                )
                mock_esi.client.Universe.post_universe_names.side_effect = (
                    esi_post_universe_names
                )
                response = create_requests.request_corp_standing(
                    request, corporation_id
                )
                success = not mock_message.called
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))
        return success

    def test_should_create_new_request_when_valid(self):
        # given
        character_1009 = create_entity(EveCharacter, 1009)
        add_character_to_user(self.user, character_1009, scopes=["publicData"])
        character_1010 = create_entity(EveCharacter, 1010)
        add_character_to_user(self.user, character_1010, scopes=["publicData"])
        # when
        result = self._view_request_corp_standing(2102)
        # then
        self.assertTrue(result)
        obj = StandingRequest.objects.get(contact_id=2102)
        self.assertFalse(obj.is_actioned)
        self.assertFalse(obj.is_effective)

    def test_should_return_false_when_not_enough_tokens(self):
        # given
        character_1009 = create_entity(EveCharacter, 1009)
        add_character_to_user(self.user, character_1009, scopes=["publicData"])
        # when
        result = self._view_request_corp_standing(2102)
        # then
        self.assertFalse(result)

    def test_should_return_false_if_pending_request(self):
        # given
        StandingRequest.objects.create(
            contact_id=2102, contact_type_id=ContactType.corporation_id, user=self.user
        )
        # when
        result = self._view_request_corp_standing(2102)
        # then
        self.assertFalse(result)

    def test_should_return_false_if_pending_revocation(self):
        # given
        StandingRevocation.objects.create(
            contact_id=2102, contact_type_id=ContactType.corporation_id, user=self.user
        )
        # when
        result = self._view_request_corp_standing(2102)
        # then
        self.assertFalse(result)


class TestRemoveCorporationStanding(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        create_contacts_set()
        create_entity(EveCharacter, 1001)
        cls.user, _ = create_user_from_evecharacter(
            1001, permissions=["standingsrequests.request_standings"]
        )

    def view_remove_corp_standing(self, corporation_id: int) -> bool:
        request = self.factory.get(
            reverse(
                "standingsrequests:remove_corp_standing",
                args=[corporation_id],
            )
        )
        request.user = self.user
        with patch(VIEWS_PATH + ".messages.warning") as mock_message:
            response = create_requests.remove_corp_standing(request, corporation_id)
            success = not mock_message.called
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("standingsrequests:create_requests"))
        return success

    def test_should_remove_valid_pending_request(self):
        # given
        character_1009 = create_entity(EveCharacter, 1009)
        add_character_to_user(self.user, character_1009, scopes=["publicData"])
        StandingRequest.objects.get_or_create_2(
            user=self.user,
            contact_id=2102,
            contact_type=StandingRequest.ContactType.CORPORATION,
        )
        # when
        success = self.view_remove_corp_standing(2102)
        # then
        self.assertTrue(success)
        self.assertFalse(StandingRequest.objects.filter(contact_id=2102).exists())

    def test_should_remove_valid_effective_request(self):
        # given
        character_1004 = create_entity(EveCharacter, 1004)
        add_character_to_user(self.user, character_1004, scopes=["publicData"])
        req = StandingRequest.objects.get_or_create_2(
            user=self.user,
            contact_id=2003,
            contact_type=StandingRequest.ContactType.CORPORATION,
        )
        req.mark_actioned(user=None)
        req.mark_effective()
        # when
        success = self.view_remove_corp_standing(2003)
        # then
        self.assertTrue(success)
        self.assertTrue(StandingRequest.objects.filter(contact_id=2003).exists())
        self.assertTrue(StandingRevocation.objects.filter(contact_id=2003).exists())

    def test_should_return_false_if_standing_requests_from_another_user(self):
        # given
        user = AuthUtils.create_member("Peter Parker")
        character_1009 = create_entity(EveCharacter, 1009)
        add_character_to_user(self.user, character_1009, scopes=["publicData"])
        StandingRequest.objects.get_or_create_2(
            user=user,
            contact_id=2102,
            contact_type=StandingRequest.ContactType.CORPORATION,
        )
        # when
        success = self.view_remove_corp_standing(2102)
        # then
        self.assertFalse(success)

    def test_should_return_false_if_no_standing_request_exists(self):
        # given
        character_1009 = create_entity(EveCharacter, 1009)
        add_character_to_user(self.user, character_1009, scopes=["publicData"])
        # when
        success = self.view_remove_corp_standing(2102)
        # then
        self.assertFalse(success)

    def test_should_return_false_if_standing_not_fully_effective(self):
        # given
        character = create_entity(EveCharacter, 1008)
        add_character_to_user(self.user, character, scopes=["publicData"])
        req = StandingRequest.objects.get_or_create_2(
            user=self.user,
            contact_id=2102,
            contact_type=StandingRequest.ContactType.CORPORATION,
        )
        req.mark_actioned(user=None)
        req.mark_effective()
        # when
        success = self.view_remove_corp_standing(2102)
        # then
        self.assertFalse(success)



import inspect
import json
import os
from copy import deepcopy
from datetime import timedelta
from unittest.mock import Mock

from bravado.exception import HTTPNotFound

from django.contrib.auth.models import User
from django.test import RequestFactory, TestCase
from django.utils.timezone import now
from eveuniverse.models import EveEntity

from allianceauth.eveonline.models import (
    EveAllianceInfo,
    EveCharacter,
    EveCorporationInfo,
)
from allianceauth.tests.auth_utils import AuthUtils
from app_utils.esi_testing import BravadoOperationStub
from app_utils.testing import add_character_to_user

from standingsrequests.core import ContactType
from standingsrequests.managers import _ContactsWrapper
from standingsrequests.models import (
    CharacterAffiliation,
    Contact,
    ContactSet,
    CorporationDetails,
    StandingRequest,
    StandingRevocation,
)

from .utils import PartialDictEqualMixin

TEST_STANDINGS_API_CHARID = 1001
TEST_STANDINGS_API_CHARNAME = "Bruce Wayne"
TEST_STANDINGS_CORPORATION_ID = 2001
TEST_STANDINGS_CORPORATION_NAME = "Wayne Technologies"
TEST_STANDINGS_ALLIANCE_ID = 3001
TEST_STANDINGS_ALLIANCE_NAME = "Wayne Enterprises"

TEST_SCOPE = "publicData"


##########################
# internal functions


def _load_test_data():
    currentdir = os.path.dirname(
        os.path.abspath(inspect.getfile(inspect.currentframe()))
    )

    with open(currentdir + "/my_test_data.json", "r", encoding="utf-8") as f:
        my_test_data = json.load(f)

    return my_test_data


def _load_entities():
    entities = dict()
    for character_id, character in _my_test_data["EveCharacter"].items():
        entities[int(character_id)] = character["character_name"]

    for corporation_id, corporation in _my_test_data["EveCorporationInfo"].items():
        entities[int(corporation_id)] = corporation["corporation_name"]

    for alliance_id, alliance in _my_test_data["EveAllianceInfo"].items():
        entities[int(alliance_id)] = alliance["alliance_name"]

    return entities


_my_test_data = _load_test_data()
_entities = _load_entities()


##########################
# common functions


def get_my_test_data() -> dict:
    """returns the raw test data dict"""
    return _my_test_data


def get_entity_name(entity_id: int):
    """returns name if entity is found, else None"""

    if int(entity_id) in _entities:
        return _entities[int(entity_id)]
    else:
        return None


def get_entity_names(eve_entity_ids: list) -> dict:
    """returns dict with {id: name} for found entities, else empty dict"""
    names_info = dict()
    for id in eve_entity_ids:
        name = get_entity_name(id)
        if name:
            names_info[id] = name

    return names_info


def get_entity_data(EntityClass: type, entity_id: int) -> object:
    if EntityClass not in [EveCharacter, EveCorporationInfo, EveAllianceInfo]:
        raise TypeError("Invalid entity_class: {}".format(EntityClass.__name__))
    if str(entity_id) not in _my_test_data[EntityClass.__name__]:
        raise ValueError(
            "not entity found in test data for that entity_id = {}".format(entity_id)
        )
    return _my_test_data[EntityClass.__name__][str(entity_id)]


def create_entity(EntityClass: type, entity_id: int) -> object:
    """creates an Eve entity from test data"""
    data = get_entity_data(EntityClass, entity_id)
    return EntityClass.objects.create(**data)


##########################
# esi emulation


def esi_post_universe_names(ids, *args, **kwargs) -> object:
    entities = list()
    for entity in _my_test_data["esi_post_universe_names"]:
        if entity["id"] in ids:
            entities.append(entity)

    return BravadoOperationStub(entities)


def esi_post_characters_affiliation(characters, *args, **kwargs) -> object:
    result = []
    for assoc in _my_test_data["CharacterAffiliation"]:
        if assoc["character_id"] in characters:
            row = assoc.copy()
            result.append(row)

    return BravadoOperationStub(result)


def esi_get_corporations_corporation_id(corporation_id, *args, **kwargs) -> object:
    result = []
    corporation_id = str(corporation_id)
    if corporation_id not in _my_test_data["EveCorporationInfo"]:
        raise HTTPNotFound(Mock(), message="Test Exception")

    row = _my_test_data["EveCorporationInfo"][corporation_id]
    result = {
        "name": row["corporation_name"],
        "ticker": row["corporation_ticker"],
        "member_count": row["member_count"],
        "ceo_id": row["ceo_id"] if "ceo_id" in row else None,
    }
    if row["alliance_id"]:
        result["alliance_id"] = row["alliance_id"]

    return BravadoOperationStub(result)


def esi_get_alliances_alliance_id_contacts_labels(*args, **kwargs) -> object:
    return BravadoOperationStub(deepcopy(_my_test_data["alliance_labels"]))


def esi_get_alliances_alliance_id_contacts(*args, **kwargs) -> object:
    return BravadoOperationStub(deepcopy(_my_test_data["alliance_contacts"]))


##########################
# app specific functions


def create_standings_char():
    character, _ = EveCharacter.objects.get_or_create(
        character_id=TEST_STANDINGS_API_CHARID,
        defaults={
            "character_name": TEST_STANDINGS_API_CHARNAME,
            "corporation_id": TEST_STANDINGS_CORPORATION_ID,
            "corporation_name": TEST_STANDINGS_CORPORATION_ID,
            "alliance_id": TEST_STANDINGS_ALLIANCE_ID,
            "alliance_name": TEST_STANDINGS_ALLIANCE_NAME,
        },
    )
    return character


def get_test_labels() -> list:
    """returns labels from test data as list of _ContactsWrapper.Label"""
    labels = list()
    for label_data in get_my_test_data()["alliance_labels"]:
        labels.append(_ContactsWrapper.Label(label_data))

    return labels


def get_test_contacts():
    """returns contacts from test data as list of _ContactsWrapper.Contact"""
    labels = get_test_labels()

    contact_ids = [x["contact_id"] for x in get_my_test_data()["alliance_contacts"]]
    names_info = get_entity_names(contact_ids)
    contacts = list()
    for contact_data in get_my_test_data()["alliance_contacts"]:
        contacts.append(_ContactsWrapper.Contact(contact_data, labels, names_info))

    return contacts


def create_contacts_set(my_set: ContactSet = None, include_assoc=True) -> ContactSet:

    if not my_set:
        my_set = ContactSet.objects.create(name="Dummy Set")

    # add labels
    ContactSet.objects._add_labels_from_api(my_set, get_test_labels())

    # create contacts for ContactSet
    for contact in _my_test_data["alliance_contacts"]:
        if contact["contact_type"] == "character":
            category = EveEntity.CATEGORY_CHARACTER
        elif contact["contact_type"] == "corporation":
            category = EveEntity.CATEGORY_CORPORATION
        elif contact["contact_type"] == "alliance":
            category = EveEntity.CATEGORY_ALLIANCE
        else:
            raise ValueError("Invalid contact type")

        eve_entity, _ = EveEntity.objects.get_or_create(
            id=contact["contact_id"],
            defaults={
                "name": get_entity_name(contact["contact_id"]),
                "category": category,
            },
        )
        my_standing = Contact.objects.create(
            contact_set=my_set,
            eve_entity=eve_entity,
            standing=contact["standing"],
        )
        for label_id in contact["label_ids"]:
            my_standing.labels.add(my_set.labels.get(label_id=label_id))

    # update EveEntity based on characters
    for character_id, character_data in _my_test_data["EveCharacter"].items():
        EveEntity.objects.get_or_create(
            id=character_id,
            defaults={
                "name": character_data["character_name"],
                "category": EveEntity.CATEGORY_CHARACTER,
            },
        )
        EveEntity.objects.get_or_create(
            id=character_data["corporation_id"],
            defaults={
                "name": character_data["corporation_name"],
                "category": EveEntity.CATEGORY_CORPORATION,
            },
        )
        if character_data["alliance_id"]:
            EveEntity.objects.get_or_create(
                id=character_data["alliance_id"],
                defaults={
                    "name": character_data["alliance_name"],
                    "category": EveEntity.CATEGORY_ALLIANCE,
                },
            )

    # create CharacterAffiliation
    if include_assoc:
        CharacterAffiliation.objects.all().delete()
        for assoc in _my_test_data["CharacterAffiliation"]:
            CharacterAffiliation.objects.create(**assoc)

    return my_set


def create_eve_objects():
    """creates all Eve objects from test data"""
    EveCharacter.objects.all().delete()
    EveCorporationInfo.objects.all().delete()
    EveAllianceInfo.objects.all().delete()
    for character_data in _my_test_data[EveCharacter.__name__].values():
        character = EveCharacter.objects.create(**character_data)
        if character.alliance_id:
            defaults = {
                "alliance_name": character.alliance_name,
                "alliance_ticker": character.alliance_ticker,
                "executor_corp_id": 2001,
            }
            alliance, _ = EveAllianceInfo.objects.get_or_create(
                alliance_id=character.alliance_id, defaults=defaults
            )
        else:
            alliance = None

        defaults = {
            "corporation_name": character.corporation_name,
            "corporation_ticker": character.corporation_ticker,
            "member_count": 99,
            "alliance": alliance,
        }
        EveCorporationInfo.objects.get_or_create(
            corporation_id=character.corporation_id, defaults=defaults
        )


def add_eve_object_to_eve_entities(obj: object):
    if isinstance(obj, EveCharacter):
        EveEntity.objects.update_or_create(
            id=obj.character_id,
            defaults={
                "name": obj.character_name,
                "category": EveEntity.CATEGORY_CHARACTER,
            },
        )
        EveEntity.objects.update_or_create(
            id=obj.corporation_id,
            defaults={
                "name": obj.corporation_name,
                "category": EveEntity.CATEGORY_CORPORATION,
            },
        )
        if obj.alliance_id:
            EveEntity.objects.update_or_create(
                id=obj.alliance_id,
                defaults={
                    "name": obj.alliance_name,
                    "category": EveEntity.CATEGORY_ALLIANCE,
                },
            )
    elif isinstance(obj, EveCorporationInfo):
        EveEntity.objects.update_or_create(
            id=obj.corporation_id,
            defaults={
                "name": obj.corporation_name,
                "category": EveEntity.CATEGORY_CORPORATION,
            },
        )
    elif isinstance(obj, EveAllianceInfo):
        EveEntity.objects.update_or_create(
            id=obj.alliance_id,
            defaults={
                "name": obj.alliance_name,
                "category": EveEntity.CATEGORY_ALLIANCE,
            },
        )
    else:
        raise NotImplementedError()


def generate_eve_entities_from_allianceauth():
    for character in EveCharacter.objects.all():
        EveEntity.objects.get_or_create(
            id=character.character_id,
            defaults={
                "name": character.character_name,
                "category": EveEntity.CATEGORY_CHARACTER,
            },
        )
        EveEntity.objects.get_or_create(
            id=character.corporation_id,
            defaults={
                "name": character.corporation_name,
                "category": EveEntity.CATEGORY_CORPORATION,
            },
        )
        if character.alliance_id:
            EveEntity.objects.get_or_create(
                id=character.alliance_id,
                defaults={
                    "name": character.alliance_name,
                    "category": EveEntity.CATEGORY_ALLIANCE,
                },
            )
    for corporation in EveCorporationInfo.objects.all():
        EveEntity.objects.get_or_create(
            id=corporation.corporation_id,
            defaults={
                "name": corporation.corporation_name,
                "category": EveEntity.CATEGORY_CORPORATION,
            },
        )
    for alliance in EveAllianceInfo.objects.all():
        EveEntity.objects.get_or_create(
            id=alliance.alliance_id,
            defaults={
                "name": alliance.alliance_name,
                "category": EveEntity.CATEGORY_ALLIANCE,
            },
        )


def load_eve_entities():
    for character in _my_test_data["EveCharacter"].values():
        EveEntity.objects.get_or_create(
            id=character["character_id"],
            defaults={
                "name": character["character_name"],
                "category": EveEntity.CATEGORY_CHARACTER,
            },
        )
        EveEntity.objects.get_or_create(
            id=character["corporation_id"],
            defaults={
                "name": character["corporation_name"],
                "category": EveEntity.CATEGORY_CORPORATION,
            },
        )
        if character["alliance_id"]:
            EveEntity.objects.get_or_create(
                id=character["alliance_id"],
                defaults={
                    "name": character["alliance_name"],
                    "category": EveEntity.CATEGORY_ALLIANCE,
                },
            )
    for corporation in _my_test_data["EveCorporationInfo"].values():
        EveEntity.objects.get_or_create(
            id=corporation["corporation_id"],
            defaults={
                "name": corporation["corporation_name"],
                "category": EveEntity.CATEGORY_CORPORATION,
            },
        )
    for alliance in _my_test_data["EveAllianceInfo"].values():
        EveEntity.objects.get_or_create(
            id=alliance["alliance_id"],
            defaults={
                "name": alliance["alliance_name"],
                "category": EveEntity.CATEGORY_ALLIANCE,
            },
        )

    for id, name in [
        (500001, "Caldari State"),
        (500002, "Minmatar Republic"),
        (500003, "Amarr Empire"),
        (500004, "Gallente Federation"),
    ]:
        EveEntity.objects.get_or_create(
            id=id,
            defaults={"name": name, "category": EveEntity.CATEGORY_FACTION},
        )


def load_corporation_details():
    for record in _my_test_data["EveCorporationInfo"].values():
        alliance_id = record["alliance_id"] if record.get("alliance_id") else None
        CorporationDetails.objects.update_or_create(
            corporation_id=record["corporation_id"],
            defaults={
                "alliance_id": alliance_id,
                "ceo_id": 2102,
                "member_count": 99,
                "ticker": record["corporation_ticker"],
            },
        )


class TestViewPagesBase(PartialDictEqualMixin, TestCase):
    """Base TestClass for all tests that deal with standing requests

    Defines common test data
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()

        create_standings_char()
        create_eve_objects()

        cls.contact_set = create_contacts_set()

        # State is alliance, all members can add standings
        member_state = AuthUtils.get_member_state()
        member_state.member_alliances.add(EveAllianceInfo.objects.get(alliance_id=3001))
        perm = AuthUtils.get_permission_by_name(StandingRequest.REQUEST_PERMISSION_NAME)
        member_state.permissions.add(perm)

        # Requesting user - can only make requests
        cls.main_character_1 = EveCharacter.objects.get(character_id=1002)
        cls.user_requestor = AuthUtils.create_member(
            cls.main_character_1.character_name
        )
        add_character_to_user(
            cls.user_requestor,
            cls.main_character_1,
            is_main=True,
            scopes=[TEST_SCOPE],
        )
        cls.alt_character_1 = EveCharacter.objects.get(character_id=1007)
        add_character_to_user(
            cls.user_requestor,
            cls.alt_character_1,
            scopes=[TEST_SCOPE],
        )
        cls.alt_corporation = EveCorporationInfo.objects.get(
            corporation_id=cls.alt_character_1.corporation_id
        )
        cls.alt_character_2 = EveCharacter.objects.get(character_id=1008)
        add_character_to_user(
            cls.user_requestor,
            cls.alt_character_2,
            scopes=[TEST_SCOPE],
        )

        # Standing manager - can do everything
        cls.main_character_2 = EveCharacter.objects.get(character_id=1001)
        cls.user_manager = AuthUtils.create_member(cls.main_character_2.character_name)
        add_character_to_user(
            cls.user_manager,
            cls.main_character_2,
            is_main=True,
            scopes=[TEST_SCOPE],
        )
        cls.user_manager = AuthUtils.add_permission_to_user_by_name(
            "standingsrequests.affect_standings", cls.user_manager
        )
        cls.user_manager = AuthUtils.add_permission_to_user_by_name(
            "standingsrequests.view", cls.user_manager
        )
        cls.user_manager = User.objects.get(pk=cls.user_manager.pk)

        # Old user - has no main and no rights
        cls.user_former_member = AuthUtils.create_user("Lex Luthor")
        cls.alt_character_3 = EveCharacter.objects.get(character_id=1010)
        add_character_to_user(
            cls.user_former_member,
            cls.alt_character_3,
            scopes=[TEST_SCOPE],
        )

    def setUp(self):
        StandingRequest.objects.all().delete()
        StandingRevocation.objects.all().delete()

    def _create_standing_for_alt(self, alt: object) -> StandingRequest:
        if isinstance(alt, EveCharacter):
            contact_id = alt.character_id
            contact_type_id = ContactType.character_id
        elif isinstance(alt, EveCorporationInfo):
            contact_id = alt.corporation_id
            contact_type_id = ContactType.corporation_id
        else:
            raise NotImplementedError()

        return StandingRequest.objects.create(
            user=self.user_requestor,
            contact_id=contact_id,
            contact_type_id=contact_type_id,
            action_by=self.user_manager,
            action_date=now() - timedelta(days=1, hours=1),
            is_effective=True,
            effective_date=now() - timedelta(days=1),
        )

    def _set_standing_for_alt_in_game(self, alt: object) -> None:
        if isinstance(alt, EveCharacter):
            contact_id = alt.character_id
            Contact.objects.update_or_create(
                contact_set=self.contact_set,
                eve_entity_id=contact_id,
                defaults={"standing": 10},
            )
        elif isinstance(alt, EveCorporationInfo):
            contact_id = alt.corporation_id
            Contact.objects.update_or_create(
                contact_set=self.contact_set,
                eve_entity_id=contact_id,
                defaults={"standing": 10},
            )
        else:
            raise NotImplementedError()

        self.contact_set.refresh_from_db()



from unittest.mock import patch

from django.test import TestCase

from allianceauth.eveonline.models import EveCharacter
from app_utils.testing import NoSocketsTestCase

from ..core import BaseConfig, ContactType
from .entity_type_ids import (
    ALLIANCE_TYPE_ID,
    CHARACTER_ACHURA_TYPE_ID,
    CHARACTER_BRUTOR_TYPE_ID,
    CHARACTER_CIVRE_TYPE_ID,
    CHARACTER_DETEIS_TYPE_ID,
    CHARACTER_DRIFTER_TYPE_ID,
    CHARACTER_GALLENTE_TYPE_ID,
    CHARACTER_INTAKI_TYPE_ID,
    CHARACTER_JIN_MEI_TYPE_ID,
    CHARACTER_KHANID_TYPE_ID,
    CHARACTER_MODIFIER_TYPE_ID,
    CHARACTER_NI_KUNNI_TYPE_ID,
    CHARACTER_SEBIESTOR_TYPE_ID,
    CHARACTER_STATIC_TYPE_ID,
    CHARACTER_TYPE_ID,
    CHARACTER_VHEROKIOR_TYPE_ID,
    CORPORATION_TYPE_ID,
)
from .my_test_data import create_entity, load_eve_entities

MODULE_PATH = "standingsrequests.core"


class TestContactType(TestCase):
    def test_get_contact_type(self):
        self.assertEqual(ContactType.character_id, CHARACTER_TYPE_ID)

    def test_is_pilot(self):
        self.assertTrue(ContactType.is_character(CHARACTER_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_NI_KUNNI_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_CIVRE_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_DETEIS_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_GALLENTE_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_INTAKI_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_SEBIESTOR_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_BRUTOR_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_STATIC_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_MODIFIER_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_ACHURA_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_JIN_MEI_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_KHANID_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_VHEROKIOR_TYPE_ID))
        self.assertTrue(ContactType.is_character(CHARACTER_DRIFTER_TYPE_ID))

        self.assertFalse(ContactType.is_character(CORPORATION_TYPE_ID))
        self.assertFalse(ContactType.is_character(ALLIANCE_TYPE_ID))
        self.assertFalse(ContactType.is_character(1))
        self.assertFalse(ContactType.is_character(None))
        self.assertFalse(ContactType.is_character(-1))
        self.assertFalse(ContactType.is_character(0))

    def test_get_contact_type_2(self):
        self.assertEqual(ContactType.corporation_id, CORPORATION_TYPE_ID)

    def test_is_corporation(self):
        self.assertTrue(ContactType.is_corporation(CORPORATION_TYPE_ID))
        self.assertFalse(ContactType.is_corporation(CHARACTER_TYPE_ID))
        self.assertFalse(ContactType.is_corporation(ALLIANCE_TYPE_ID))
        self.assertFalse(ContactType.is_corporation(1))
        self.assertFalse(ContactType.is_corporation(None))
        self.assertFalse(ContactType.is_corporation(-1))
        self.assertFalse(ContactType.is_corporation(0))

    def test_get_contact_type_3(self):
        self.assertEqual(ContactType.alliance_id, ALLIANCE_TYPE_ID)

    def test_is_alliance(self):
        self.assertTrue(ContactType.is_alliance(ALLIANCE_TYPE_ID))
        self.assertFalse(ContactType.is_alliance(CHARACTER_TYPE_ID))
        self.assertFalse(ContactType.is_alliance(CORPORATION_TYPE_ID))
        self.assertFalse(ContactType.is_alliance(1))
        self.assertFalse(ContactType.is_alliance(None))
        self.assertFalse(ContactType.is_alliance(-1))
        self.assertFalse(ContactType.is_alliance(0))


@patch(MODULE_PATH + ".STANDINGS_API_CHARID", 1001)
class TestBaseConfig(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_eve_entities()

    def test_should_return_existing_character(self):
        # given
        character = create_entity(EveCharacter, 1001)
        # when
        owner_character = BaseConfig.owner_character()
        # then
        self.assertEqual(character, owner_character)

    @patch(MODULE_PATH + ".EveCharacter.objects.create_character")
    def test_create_new_character_if_not_exists(self, mock_create_character):
        # given
        character = create_entity(EveCharacter, 1002)
        mock_create_character.return_value = character
        # when
        owner_character = BaseConfig.owner_character()
        # then
        self.assertEqual(character, owner_character)

    @patch(MODULE_PATH + ".SR_OPERATION_MODE", "alliance")
    def test_should_return_alliance(self):
        # given
        create_entity(EveCharacter, 1001)
        # when
        result = BaseConfig.standings_source_entity()
        # then
        self.assertEqual(result.id, 3001)

    @patch(MODULE_PATH + ".SR_OPERATION_MODE", "corporation")
    def test_should_return_corporation(self):
        # given
        create_entity(EveCharacter, 1001)
        # when
        result = BaseConfig.standings_source_entity()
        # then
        self.assertEqual(result.id, 2001)



from datetime import timedelta
from unittest.mock import patch

from django.test import override_settings
from django.utils.timezone import now

from app_utils.testing import NoSocketsTestCase

from .. import tasks
from ..models import ContactSet
from .my_test_data import create_contacts_set

MODULE_PATH = "standingsrequests.tasks"


@patch(MODULE_PATH + ".StandingRequest.objects.process_requests")
@patch(MODULE_PATH + ".StandingRevocation.objects.process_requests")
@patch(MODULE_PATH + ".ContactSet.objects.create_new_from_api")
class TestStandingsUpdate(NoSocketsTestCase):
    def test_can_update_standings(
        self,
        mock_create_new_from_api,
        mock_requests_process_standings,
        mock_revocations_process_standings,
    ):
        tasks.standings_update()
        self.assertTrue(mock_create_new_from_api.called)
        self.assertTrue(mock_requests_process_standings.called)
        self.assertTrue(mock_revocations_process_standings.called)

    def test_can_handle_api_error(
        self,
        mock_create_new_from_api,
        mock_requests_process_standings,
        mock_revocations_process_standings,
    ):
        mock_create_new_from_api.return_value = None
        tasks.standings_update()
        self.assertTrue(mock_create_new_from_api.called)
        self.assertFalse(mock_requests_process_standings.called)
        self.assertFalse(mock_revocations_process_standings.called)


class TestOtherTasks(NoSocketsTestCase):
    @patch(MODULE_PATH + ".StandingRequest.objects.validate_requests")
    def test_validate_standings_requests(self, mock_validate_standings_requests):
        tasks.validate_requests()
        self.assertTrue(mock_validate_standings_requests.called)

    @override_settings(
        CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True
    )
    @patch(MODULE_PATH + ".CorporationDetails.objects.update_or_create_from_esi")
    @patch(MODULE_PATH + ".CharacterAffiliation.objects.update_evecharacter_relations")
    @patch(MODULE_PATH + ".CharacterAffiliation.objects.update_from_esi")
    def test_update_associations_api(
        self,
        mock_update_from_esi,
        mock_update_evecharacter_relations,
        mock_update_or_create_from_esi,
    ):
        # when
        create_contacts_set()
        tasks.update_associations_api.delay()
        # then
        self.assertTrue(mock_update_from_esi.called)
        self.assertTrue(mock_update_evecharacter_relations.called)
        self.assertTrue(mock_update_or_create_from_esi.called)


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestPurgeTasks(NoSocketsTestCase):
    @patch(MODULE_PATH + ".purge_stale_standings_data")
    def test_purge_stale_data(self, mock_purge_stale_standings_data):
        tasks.purge_stale_data.delay()
        self.assertTrue(mock_purge_stale_standings_data.si.called)


@patch(MODULE_PATH + ".SR_STANDINGS_STALE_HOURS", 48)
class TestPurgeStaleStandingData(NoSocketsTestCase):
    def setUp(self):
        ContactSet.objects.all().delete()

    def test_do_nothing_if_not_contacts_sets(self):
        tasks.purge_stale_standings_data()

    def test_one_younger_set_no_purge(self):
        set_1 = create_contacts_set()
        tasks.purge_stale_standings_data()
        current_pks = set(ContactSet.objects.values_list("pk", flat=True))
        expected = {set_1.pk}
        self.assertSetEqual(current_pks, expected)

    def test_one_older_set_no_purge(self):
        set_1 = create_contacts_set()
        set_1.date = now() - timedelta(hours=48, seconds=1)
        set_1.save()
        tasks.purge_stale_standings_data()
        current_pks = set(ContactSet.objects.values_list("pk", flat=True))
        expected = {set_1.pk}
        self.assertSetEqual(current_pks, expected)

    def test_two_younger_sets_no_purge(self):
        set_1 = create_contacts_set()
        set_2 = create_contacts_set()
        tasks.purge_stale_standings_data()
        current_pks = set(ContactSet.objects.values_list("pk", flat=True))
        expected = {set_1.pk, set_2.pk}
        self.assertSetEqual(current_pks, expected)

    def test_two_sets_young_and_old_purge_older_only(self):
        set_1 = create_contacts_set()
        set_1.date = now() - timedelta(hours=48, seconds=1)
        set_1.save()
        set_2 = create_contacts_set()
        tasks.purge_stale_standings_data()
        current_pks = set(ContactSet.objects.values_list("pk", flat=True))
        expected = {set_2.pk}
        self.assertSetEqual(current_pks, expected)

    def test_two_older_set_purge_older_one_only(self):
        set_1 = create_contacts_set()
        set_1.date = now() - timedelta(hours=48, seconds=2)
        set_1.save()
        set_2 = create_contacts_set()
        set_1.date = now() - timedelta(hours=48, seconds=1)
        set_1.save()
        tasks.purge_stale_standings_data()
        current_pks = set(ContactSet.objects.values_list("pk", flat=True))
        expected = {set_2.pk}
        self.assertSetEqual(current_pks, expected)


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestUpdateAllCorporationDetails(NoSocketsTestCase):
    def setUp(self):
        create_contacts_set()

    @patch(MODULE_PATH + ".CorporationDetails.objects.update_or_create_from_esi")
    def test_should_update_all_corporation_details(
        self, mock_update_or_create_from_esi
    ):
        # when
        tasks.update_all_corporation_details.delay()
        # then
        called_corporation_ids = {
            obj[0][0] for obj in mock_update_or_create_from_esi.call_args_list
        }
        self.assertSetEqual(called_corporation_ids, {2001, 2003, 2004, 2102})
